@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: BuildMessage.py,v 1.65 2003/12/08 02:22:56 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: BuildMessage.py,v 1.73 2004/03/23 00:07:02 nickm Exp $
 
 """mixminion.BuildMessage
 
@@ -89,7 +89,7 @@ def buildRandomPayload(paddingPRNG=None):
     return paddingPRNG.getBytes(PAYLOAD_LEN)
 
 def buildForwardPacket(payload, exitType, exitInfo, path1, path2,
-                        paddingPRNG=None):
+                       paddingPRNG=None, suppressTag=0):
     """Construct a forward message.
             payload: The payload to deliver.  Must be exactly 28K.  If the
                   payload is None, 28K of random data is sent.
@@ -99,6 +99,8 @@ def buildForwardPacket(payload, exitType, exitInfo, path1, path2,
             path2: Sequence of ServerInfo objects for the 2nd leg of the path
             paddingPRNG: random number generator used to generate padding.
                   If None, a new PRNG is initialized.
+            suppressTag: if true, do not include a decodind handle in the
+                  routingInfo for this packet.
 
         Neither path1 nor path2 may be empty.  If one is, MixError is raised.
     """
@@ -109,11 +111,6 @@ def buildForwardPacket(payload, exitType, exitInfo, path1, path2,
     if not path2:
         raise MixError("Second leg of path is empty")
 
-    suppressTag = 0
-    #XXXX refactor _TYPES_WITHOUT_TAGS
-    if exitType == DROP_TYPE or mixminion.Packet._TYPES_WITHOUT_TAGS.get(exitType):
-        suppressTag = 1
-
     assert len(payload) == PAYLOAD_LEN
 
     LOG.debug("  Using path %s:%s",
@@ -126,7 +123,7 @@ def buildForwardPacket(payload, exitType, exitInfo, path1, path2,
         tag = _getRandomTag(paddingPRNG)
         exitInfo = tag + exitInfo
     return _buildPacket(payload, exitType, exitInfo, path1, path2,
-                        paddingPRNG)
+                        paddingPRNG,suppressTag=suppressTag)
 
 
 def buildEncryptedForwardPacket(payload, exitType, exitInfo, path1, path2,
@@ -260,6 +257,7 @@ def _buildReplyBlockImpl(path, exitType, exitInfo, expiryTime=0,
 
     # XXXX007 switch to Host info.  We need to use IPV4 for reply blocks
     # XXXX007 for now, since we don't know which servers will support HOST.
+    # XXXX007    (Do this after all hosts have upgraded to 0.0.6 or later.)
     return ReplyBlock(header, expiryTime,
                       SWAP_FWD_IPV4_TYPE,
                       path[0].getIPV4Info().pack(), sharedKey), secrets, tag
@@ -301,11 +299,14 @@ def buildReplyBlock(path, exitType, exitInfo, userKey,
     return _buildReplyBlockImpl(path, exitType, exitInfo, expiryTime, prng,
                                 seed)[0]
 
-def checkPathLength(path1, path2, exitType, exitInfo, explicitSwap=0):
+def checkPathLength(path1, path2, exitType, exitInfo, explicitSwap=0,
+                    suppressTag=0):
     """Given two path legs, an exit type and an exitInfo, raise an error
-       if we can't build a hop with the provided legs.
+       if we can't build a hop with the provided legs.  If suppressTag is
+       true, no decoding handle will be included.
 
-       The leg "path1" may be null."""
+       The leg "path1" may be null.
+    """
     err = 0 # 0: no error. 1: 1st leg too big. 2: 1st leg okay, 2nd too big.
     if path1 is not None:
         try:
@@ -314,8 +315,7 @@ def checkPathLength(path1, path2, exitType, exitInfo, explicitSwap=0):
         except MixError:
             err = 1
     # Add a dummy tag as needed to last exitinfo.
-    if (exitType != DROP_TYPE
-        and not mixminion.Packet._TYPES_WITHOUT_TAGS.get(exitType)
+    if (not suppressTag
         and exitInfo is not None):
         exitInfo += "X"*20
     else:
@@ -333,7 +333,7 @@ def checkPathLength(path1, path2, exitType, exitInfo, explicitSwap=0):
 
 #----------------------------------------------------------------------
 # MESSAGE DECODING
-def decodePayload(payload, tag, key=None, userKeys=()):
+def decodePayload(payload, tag, key=None, userKeys=(), retNym=None):
     """Given a 28K payload and a 20-byte decoding tag, attempt to decode the
        original message.  Returns either a SingletonPayload instance, a
        FragmentPayload instance, or None.
@@ -343,6 +343,9 @@ def decodePayload(payload, tag, key=None, userKeys=()):
                 SURB keys. For backward compatibility, 'userKeys' may also be
                 None (no SURBs known), a dict (from name to key), or a single
                 key (implied identity is "").
+           retNym: If present, and if the payload was a reply, we call
+                retNym.append(pseudonym).  (For the default SURB identity,
+                we append the empty string.)
 
        If we can successfully decrypt the payload, we return it.  If we
        might be able to decrypt the payload given more/different keys,
@@ -378,6 +381,8 @@ def decodePayload(payload, tag, key=None, userKeys=()):
                 p = _decodeStatelessReplyPayload(payload, tag, userKey)
                 if name:
                     LOG.info("Decoded reply message to identity %r", name)
+                if retNym is not None:
+                    retNym.append(name)
                 return p
             except MixError:
                 pass
@@ -462,7 +467,8 @@ def _decodeStatelessReplyPayload(payload, tag, userKey):
 
 #----------------------------------------------------------------------
 def _buildPacket(payload, exitType, exitInfo,
-                 path1, path2, paddingPRNG=None, paranoia=0):
+                 path1, path2, paddingPRNG=None, paranoia=0,
+                 suppressTag=0):
     """Helper method to create a message.
 
     The following fields must be set:
@@ -494,7 +500,7 @@ def _buildPacket(payload, exitType, exitInfo,
         reply = path2
         path2 = None
     else:
-        if len(exitInfo) < TAG_LEN and exitType != DROP_TYPE and not mixminion.Packet._TYPES_WITHOUT_TAGS.get(exitType):
+        if len(exitInfo) < TAG_LEN and not suppressTag:
             raise MixError("Implausibly short exit info: %r"%exitInfo)
         if exitType < MIN_EXIT_TYPE and exitType != DROP_TYPE:
             raise MixError("Invalid exit type: %4x"%exitType)
@@ -543,11 +549,15 @@ def _buildHeader(path,secrets,exitType,exitInfo,paddingPRNG):
                each of the subheaders.
            exitType: The routing type for the last node in the header
            exitInfo: The routing info for the last node in the header.
-               (Must include 20-byte decoding tag.)
+               (Must include 20-byte decoding tag, if any.)
            paddingPRNG: A pseudo-random number generator to generate padding
     """
     assert len(path) == len(secrets)
 
+    for info in path:
+        if not info.supportsPacketVersion():
+            raise MixError("Server %s does not support any recognized packet format."%info.getNickname())
+
     routing, sizes, totalSize = _getRouting(path, exitType, exitInfo)
     if totalSize > HEADER_LEN:
         raise MixError("Path cannot fit in header")
@@ -688,7 +698,7 @@ def _getRandomTag(rng):
 
 def _checkPayload(payload):
     'Return true iff the hash on the given payload seems valid'
-    if ord(payload[0]) & 0x80:
+    if (ord(payload[0]) & 0x80):
         return payload[3:23] == Crypto.sha1(payload[23:])
     else:
         return payload[2:22] == Crypto.sha1(payload[22:])
