@@ -1,4 +1,4 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
 # Id: ClientMain.py,v 1.89 2003/06/05 18:41:40 nickm Exp $
 
 """mixminion.ClientUtils
@@ -8,7 +8,8 @@
    """
 
 __all__ = [ 'NoPassword', 'PasswordManager', 'getPassword_term',
-            'getNewPassword_term', 'SURBLog', 'ClientQueue' ]
+            'getNewPassword_term', 'SURBLog', 'ClientQueue',
+            'ClientFragmentPool' ]
 
 import binascii
 import cPickle
@@ -16,16 +17,17 @@ import getpass
 import os
 import sys
 import time
+import types
 import struct
 
 import mixminion.Filestore
+import mixminion.Packet
 
 from mixminion.Common import LOG, MixError, UIError, ceilDiv, \
      createPrivateDir, floorDiv, previousMidnight, readFile, \
-     succeedingMidnight, writeFile, armorText, unarmorText
+     succeedingMidnight, writeFile, armorText, unarmorText, MixFatalError
 from mixminion.Crypto import sha1, ctr_crypt, DIGEST_LEN, AES_KEY_LEN, \
      getCommonPRNG, trng
-from mixminion.ServerInfo import displayServer
 
 #----------------------------------------------------------------------
 class BadPassword(MixError):
@@ -39,6 +41,9 @@ class PasswordManager:
        abstract class."""
     ## Fields
     # passwords: map from password name to string value of the password.
+    # do_retry: static field: should we keep asking for a password until
+    #    one is correct?
+    do_retry = 1
     def __init__(self):
         """Create a new PasswordManager"""
         self.passwords = {}
@@ -83,6 +88,8 @@ class PasswordManager:
             if confirmFn(pwd):
                 self.passwords[name] = pwd
                 return pwd
+            if not self.do_retry:
+                break
             maxTries -= 1
             pmt = "Incorrect password. "+prompt
 
@@ -95,13 +102,37 @@ class PasswordManager:
 class CLIPasswordManager(PasswordManager):
     """Impementation of PasswordManager that asks for passwords from the
        command line."""
-    def __init__(self):
+    def __init__(self, password_fileno=None):
         PasswordManager.__init__(self)
     def _getPassword(self, name, prompt):
         return getPassword_term(prompt)
     def _getNewPassword(self, name, prompt):
         return getNewPassword_term(prompt)
 
+class FDPasswordManager(PasswordManager):
+    """Impementation of PasswordManager that asks for passwords from a
+       specified fileno."""
+    do_retry = 0
+    def __init__(self, password_fileno=None):
+        PasswordManager.__init__(self)
+        self.password_fileno = password_fileno
+    def _getPassword(self, name, prompt):
+        return getPassword_fd(self.password_fileno)
+    def _getNewPassword(self, name, prompt):
+        return getPassword_fd(self.password_fileno)
+
+def getPassword_fd(fileno):
+    """Read a password from a specified fileno."""
+    pw = ""
+    while 1:
+        chunk = os.read(fileno, 1024) # read from --password-fd filehandle
+        if not chunk:
+            break
+        pw += chunk
+    # Strip trailing endline from password, if any.
+    if pw.endswith("\n"): pw = pw[:-1]
+    return pw
+
 def getPassword_term(prompt):
     """Read a password from the console, then return it.  Use the string
        'message' as a prompt."""
@@ -152,6 +183,8 @@ def getNewPassword_term(prompt):
         f.write("Passphrases do not match.\n")
         f.flush()
 
+    raise AssertionError # unreached; appease pychecker
+
 #----------------------------------------------------------------------
 # Functions to save and load data do disk in password-encrypted files.
 #
@@ -193,8 +226,8 @@ def _readEncryptedFile(fname, password, magicList):
     key = sha1(salt+password+salt)[:AES_KEY_LEN]
     s = ctr_crypt(s, key)
     data = s[:-DIGEST_LEN]
-    hash = s[-DIGEST_LEN:]
-    if hash != sha1(data+salt+magic):
+    digest = s[-DIGEST_LEN:]
+    if digest != sha1(data+salt+magic):
         raise BadPassword()
 
     # We've decrypted it; now let's extract the data from the padding.
@@ -218,8 +251,8 @@ def _writeEncryptedFile(fname, password, magic, data):
     data = "".join([length,data,padding])
     salt = prng.getBytes(SALT_LEN)
     key = sha1(salt+password+salt)[:AES_KEY_LEN]
-    hash = sha1("".join([data,salt,magic]))
-    encrypted = ctr_crypt(data+hash, key)
+    digest = sha1("".join([data,salt,magic]))
+    encrypted = ctr_crypt(data+digest, key)
     contents = "".join([magic,"\x00",salt,encrypted])
     writeFile(fname, armorText(contents,
                                "TYPE III KEYRING", [("Version","0.1")]))
@@ -374,9 +407,9 @@ class _KeyringImpl:
     # surbKeys: A map from lowercase keyid to a list of (expiry-time, secret)
     #    for all of the SURB keys in the keyring.
     SURB_KEY_TYPE = 0x00
-    def __init__(self, input="", now=None):
+    def __init__(self, s="", now=None):
         """Initialize this keyring representation from the encoded string
-           'input'.  If any keys are set to expire before 'now', delete them.
+           's'.  If any keys are set to expire before 'now', delete them.
         """
         if now is None: now = time.time()
 
@@ -384,18 +417,18 @@ class _KeyringImpl:
         self.unrecognized = []
         rec = []
         self.dirty = 0
-        while input:
-            if len(input) < 3:
+        while s:
+            if len(s) < 3:
                 raise MixError("Corrupt keyring: truncated entry.")
-            tp,length = struct.unpack("!BH", input[:3])
-            if len(input) < 3+length:
+            tp,length = struct.unpack("!BH", s[:3])
+            if len(s) < 3+length:
                 raise MixError("Corrupt keyring: truncated entry.")
-            val = input[3:3+length]
+            val = s[3:3+length]
             if tp == self.SURB_KEY_TYPE:
                 rec.append((tp,val))
             else:
                 self.unrecognized.append((tp,val))
-            input = input[3+length:]
+            s = s[3+length:]
 
         # Now, extract all the SURB keys from the keyring, and remove all
         # expired SURB keys from self.recognized.
@@ -524,7 +557,6 @@ class Keyring(_LazyEncryptedStore):
             del self._now
 
 # ----------------------------------------------------------------------
-
 class SURBLog(mixminion.Filestore.DBBase):
     """A SURBLog manipulates a database on disk to remember which SURBs we've
        used, so we don't reuse them accidentally.
@@ -605,12 +637,12 @@ class SURBLog(mixminion.Filestore.DBBase):
             now = time.time() + 60*60
         allHashes = self.log.keys()
         removed = []
-        for hash in allHashes:
-            if self._decodeVal(self.log[hash]) < now:
-                removed.append(hash)
+        for h in allHashes:
+            if self._decodeVal(self.log[h]) < now:
+                removed.append(h)
         del allHashes
-        for hash in removed:
-            del self.log[hash]
+        for h in removed:
+            del self.log[h]
         self.log['LAST_CLEANED'] = str(int(now))
         self.sync()
 
@@ -657,7 +689,7 @@ class ClientQueue:
         createPrivateDir(directory)
 
         # We used to name entries "pkt_X"; this has changed.
-        # XXXX007 remove this when it's no longer needed.
+        # XXXX008a remove this when it's no longer needed.
         for fn in os.listdir(directory):
             if fn.startswith("pkt_"):
                 handle = fn[4:]
@@ -692,11 +724,80 @@ class ClientQueue:
         finally:
             mixminion.ClientMain.clientUnlock()
 
+    def getHandlesByAge(self, notAfter):
+        """Return a list of all handles for messages that were inserted into
+           the queue before 'notAfter'."""
+        self.loadMetadata()
+        result = []
+        for h in self.store.getAllMessages():
+            _,_,when = self.store.getMetadata(h)
+            if when <= notAfter: result.append(h)
+        return result
+
+    def getHandlesByDestAndAge(self, destList, directory, notAfter=None,
+                               warnUnused=1):
+        """Return a list of handles for all messages queued for servers in a
+           given list before a given date.
+
+              destList -- A list of hostnames, ips, keyids, or nicknames
+                for servers whose messages should be included in the result.
+              directory -- An instance of ClientDirectory used to resolve
+                nicknames.  This may be None if no nicknames are included.
+              notAfter -- If provided, a time such that no messages queued
+                later should be included
+              warnUnused -- If true, we log a message for every element in
+                destList that has no matching messages in the queue.
+        """
+        destSet = {}
+        reverse = {}
+        for d in destList:
+            if directory:
+                keyid = directory.getKeyIDByNickname(d)
+                if keyid:
+                    destSet[keyid] = 1
+                    reverse[keyid] = d
+                    continue
+            destSet[d] = 1
+
+        self.loadMetadata()
+        result = []
+        foundAny = {}
+        foundMatch = {}
+        for h in self.store.getAllMessages():
+            _, r, when = self.store.getMetadata(h)
+            if (destSet.has_key(r.keyinfo) or
+                (hasattr(r, 'hostname') and destSet.has_key(r.hostname)) or
+                (hasattr(r, 'ip') and destSet.has_key(r.ip))):
+
+                keys = [ getattr(r, 'hostname', None),
+                         getattr(r, 'ip', None),
+                         reverse.get(r.keyinfo, None),
+                         r.keyinfo ]
+                for k in keys: foundAny[k]=1
+                if notAfter and when > notAfter:
+                    continue
+                for k in keys: foundMatch[k]=1
+                result.append(h)
+        if warnUnused:
+            for d in destList:
+                if foundMatch.get(d):
+                    continue
+                elif foundAny.get(d):
+                    LOG.warn("No expired packets found for %r", d)
+                else:
+                    LOG.warn("No pending packets found for %r", d)
+        return result
+
     def getRouting(self, handle):
         """Return the routing information associated with the given handle."""
         self.loadMetadata()
         return self.store.getMetadata(handle)[1]
 
+    def getDate(self, handle):
+        """Return the date a given handle was inserted."""
+        self.loadMetadata()
+        return self.store.getMetadata(handle)[2]
+
     def getPacket(self, handle):
         """Given a handle, return a 3-tuple of the corresponding
            32K packet, {IPV4/Host}Info, and time of first queueing.  (The time
@@ -746,24 +847,8 @@ class ClientQueue:
             res[s] = (count, oldest)
         return res
 
-    def cleanQueue(self, maxAge=None, now=None):
+    def cleanQueue(self):
         """Remove all packets older than maxAge seconds from this queue."""
-        if now is None:
-            now = time.time()
-        if maxAge is not None:
-            cutoff = now - maxAge
-            remove = []
-            self.loadMetadata()
-            for h in self.getHandles():
-                try:
-                    when = self.store.getMetadata(h)[2]
-                except mixminion.Filestore.CorruptedFile:
-                    continue
-                if when < cutoff:
-                    remove.append(h)
-            LOG.info("Removing %s old packets from queue", len(remove))
-            for h in remove:
-                self.store.removeMessage(h)
         self.store.cleanQueue()
         self.store.cleanMetadata()
 
@@ -777,6 +862,130 @@ class ClientQueue:
             packet, routing, when = self.getPacket(h)
             return "V0", routing, when
 
-        self.store.loadAllMetadata(fixupHandle)
+        mixminion.ClientMain.clientLock()
+        try:
+            self.store.loadAllMetadata(fixupHandle)
+        finally:
+            mixminion.ClientMain.clientUnlock()
 
         self.metadataLoaded = 1
+
+# ----------------------------------------------------------------------
+
+class ClientFragmentPool:
+    """Wrapper around FragmentPool to provide a good interface for client-side
+       fragment reassembly."""
+    def __init__(self, directory):
+        """Create a new FragmentPool storing its messages in 'directory'."""
+        createPrivateDir(directory)
+        self.dir = directory
+        self.pool = None
+
+    def __getPool(self):
+        """Helper: initialize self.pool and return it."""
+        if self.pool is None:
+            import mixminion.Fragments
+            self.pool = mixminion.Fragments.FragmentPool(self.dir)
+        return self.pool
+
+    def close(self):
+        """Finalize self.pool."""
+        if self.pool is not None:
+            self.pool.close()
+            self.pool = None
+
+    def addFragment(self, fragment, nym=None):
+        """Add a fragment to the pool, logging appropriate messages.  Return
+           the messageID which was updated, if any.
+
+             fragment -- an instance of FragmentPayload or a string payload.
+             nym -- the identity which received this message.
+        """
+        pool = self.__getPool()
+        if isinstance(fragment, types.StringType):
+            try:
+                fragment = mixminion.Packet.parsePayload(fragment)
+            except mixminion.Packet.ParseError, s:
+                raise UIError("Corrupted fragment payload: %s"%s)
+            if fragment.isSingleton():
+                raise UIError("Non-fragment payload marked as a fragment.")
+
+        assert isinstance(fragment, mixminion.Packet.FragmentPayload)
+
+        r = pool.addFragment(fragment, nym=nym, verbose=1)
+        return r
+
+    def process(self):
+        """Unchunk any messages that are ready for reassembly."""
+        pool = self.__getPool()
+        pool.unchunkMessages()
+        pool.cleanQueue()
+
+    def expireMessages(self, cutoff):
+        """Clean up any stale fragments from the pool that have been there
+           since before 'cutoff'."""
+        pool = self.__getPool()
+        pool.expireMessages(cutoff)
+        self.cleanQueue()
+
+    def getMessage(self, msgid, force=0):
+        """Return the string value of the (compressed) reassembled
+           message with ID 'msgid', or raise an error explaining why
+           we can't.
+
+           If 'force' is true, return the message even if it seems
+           overcompressed.  Otherwise raise a CompressedDataTooLong
+           exception.
+        """
+        pool = self.__getPool()
+        state = pool.getStateByMsgID(msgid)
+        if state is not None:
+            msg = pool.getReadyMessage(state.messageid)
+            if msg is not None:
+                try:
+                    if force:
+                        maxSize = None
+                    else:
+                        maxSize = len(msg)*20
+                    return mixminion.Packet.uncompressData(msg,maxSize)
+                except mixminion.Packet.ParseError, e:
+                    raise UIError("Invalid message %s: %s"%(msgid,e))
+
+        if state is None:
+            raise UIError("No such message as '%s'" % msgid)
+        elif not state.isDone():
+            raise UIError("Message '%s' is still missing fragments."%msgid)
+        else:
+            raise MixFatalError("Can't decode message %s; I don't know why!"
+                                %msgid)
+
+    def removeMessages(self, msgids):
+        """Remove all the messages whose IDs are in the list 'msgIDs'.  If the
+           messages were reassembled, mark them as 'COMPLETED'; else mark them
+           as 'REJECTED'."""
+        pool = self.__getPool()
+        idSet = {}
+        for i in msgids:
+            state = pool.getStateByMsgID(i)
+            if state is None:
+                raise UIError("No such message as %s")
+            idSet[state.messageid] = 1
+        pool._deleteMessageIDs(idSet, "?")
+        pool.cleanQueue()
+
+    def listMessages(self):
+        """Return a list of pretty-printed IDs for the messages in the pool."""
+        pool = self.__getPool()
+        return pool.listMessages()
+
+    def formatMessageList(self):
+        """Return a list of strings suitable for display explaining the status
+           of the messages in the pool, sorted by pretty-printed ID."""
+        msgs = self.listMessages()
+        result = []
+        msgids = msgs.keys()
+        msgids.sort()
+        for msgid in msgids:
+            result.append(msgid+(": to <%(nym)s>. %(size)s bytes (%(have)s/%(need)s packets received)"
+                                 % msgs[msgid]))
+        return result
