@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: Common.py,v 1.124.2.1 2003/12/18 23:13:55 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: Common.py,v 1.137 2004/03/23 05:15:16 nickm Exp $
 
 """mixminion.Common
 
@@ -33,13 +33,6 @@ import sys
 import threading
 import time
 import traceback
-# Imported here so we can get it in mixminion.server without being shadowed
-# by the old Queue.py file.
-from Queue import Queue, Empty
-MessageQueue = Queue
-QueueEmpty = Empty
-del Queue
-del Empty
 
 O_BINARY = getattr(os, 'O_BINARY', 0)
 
@@ -264,10 +257,10 @@ def armorText(s, type, headers=(), base64=1):
     return "".join(result)
 
 # Matches a begin line.
-BEGIN_LINE_RE = re.compile(r'^-----BEGIN ([^-]+)-----[ \t]*$',re.M)
+BEGIN_LINE_RE = re.compile(r'^-----BEGIN ([^-]+)-----[ \t]*\r?$',re.M)
 
 # Matches a header line.
-ARMOR_KV_RE = re.compile(r'([^:\s]+): ([^\n]+)')
+ARMOR_KV_RE = re.compile(r'([^:\s]+): ([^\r\n]+)')
 def unarmorText(s, findTypes, base64=1, base64fn=None):
     """Parse a list of OpenPGP-style ASCII-armored messages from 's',
        and return a list of (type, headers, body) tuples, where 'headers'
@@ -291,7 +284,7 @@ def unarmorText(s, findTypes, base64=1, base64fn=None):
             return result
 
         tp = mBegin.group(1)
-        endPat = r"^-----END %s-----[ \t]*$" % tp
+        endPat = r"^-----END %s-----[ \t]*\r?$" % tp
 
         endRE = re.compile(endPat, re.M)
         mEnd = endRE.search(s, mBegin.start())
@@ -323,8 +316,8 @@ def unarmorText(s, findTypes, base64=1, base64fn=None):
 
         if base64:
             try:
-                if stringContains(s[idx:endIdx], "\n[...]\n"):
-                    raise ValueError("Value seems to be truncated by a Mixminion-Mixmaster gateway")
+                if stringContains(s[idx:endIdx], "\n[...]"):
+                    raise UIError("Corrupted data: value seems to be truncated by a Mixminion/Mixmaster gateway")
                 value = binascii.a2b_base64(s[idx:endIdx])
             except (TypeError, binascii.Incomplete, binascii.Error), e:
                 raise ValueError(str(e))
@@ -577,7 +570,7 @@ def readFile(fn, binary=0):
     finally:
         f.close()
 
-def writeFile(fn, contents, mode=0600, binary=0):
+def writeFile(fn, contents, mode=0600, binary=0, fsync=0):
     """Atomically write a string <contents> into a file <file> with mode
        <mode>.  If <binary>, binary mode will be used.
 
@@ -585,12 +578,16 @@ def writeFile(fn, contents, mode=0600, binary=0):
 
        If two processes attempt to writeFile the same file at once,
        the one finishing last wins.
+
+       If fsync is true, we call fsync on the file before closing it.
        """
     tmpname = fn+".tmp"
     f, tmpname = openUnique(tmpname, ['w','wb'][binary], mode)
     try:
         try:
             f.write(contents)
+            if fsync:
+                os.fsync(f.fileno())
         finally:
             f.close()
     except:
@@ -755,14 +752,25 @@ def secureDelete(fnames, blocking=0):
     # Some systems are unhappy when you call them with too many options.
     for i in xrange(0, len(fnames), 250-len(_SHRED_OPTS)):
         files = fnames[i:i+250-len(_SHRED_OPTS)]
-        pid = os.spawnl(os.P_NOWAIT,
-                        _SHRED_CMD, _SHRED_CMD, *(_SHRED_OPTS+files))
-        if blocking:
-            try:
-                os.waitpid(pid, 0)
-            except OSError:
-                # sigchild handler might get to the pid first.
-                pass
+        try:
+            #XXXX008 if blocking, we should just call this with P_WAIT.
+            pid = os.spawnl(os.P_NOWAIT,
+                            _SHRED_CMD, _SHRED_CMD, *(_SHRED_OPTS+files))
+        except OSError, e:
+            if e.errno not in (errno.EAGAIN, errno.ENOMEM):
+                raise
+            LOG.warn("Transient error while shredding files: %s",e)
+            for f in files:
+                if os.path.exists(f):
+                    _overwriteFile(f)
+                    os.unlink(f)
+        else:
+            if blocking:
+                try:
+                    os.waitpid(pid, 0)
+                except OSError:
+                    # sigchild handler might get to the pid first.
+                    pass
 
 #----------------------------------------------------------------------
 # Logging
@@ -770,11 +778,33 @@ def secureDelete(fnames, blocking=0):
 # I'm trying to make this interface look like a subset of the one in
 # the draft PEP-0282 (http://www.python.org/peps/pep-0282.html).
 
-def _logtime():
+# This weirdness is used to memoize lookups in the 'time' module's namespace.
+# "LOG.log" can get called a lot, so this matters.
+def _logtime(_time=time.time,_strftime=time.strftime,
+             _localtime=time.localtime,_tzadj=[None],_dst=[None]):
     'Helper function.  Returns current local time formatted for log.'
-    t = time.time()
-    return "%s.%03d"%(time.strftime("%b %d %H:%M:%S", time.localtime(t)),
-                      (t*1000)%1000)
+    t = _time()
+    lt = _localtime(t)
+
+    # We use the '_dst[0]' variable to check whether our DST setting
+    # has changed since the last time _logtime was called.
+    if lt[8]!=_dst[0]:
+        # If it was, we regenerate the string '_tzadj[0]' to be the
+        # adjustment to UTC used to get local time.
+        _dst[0]=lt[8]
+        offset = floorDiv((calendar.timegm(lt)-time.mktime(lt)),60)
+        if offset>=0:
+            sign="+"
+        else:
+            sign='-'
+        h,m=divmod(abs(offset),60)
+        _tzadj[0]="%s%02d%02d"%(sign,h,m)
+
+    return "%s.%03d %s"%(_strftime("%b %d %H:%M:%S", lt),
+                         (t*1000)%1000,
+                         _tzadj[0])
+    #return time.strftime("%b %d %H:%M:%S.%%03d %z", time.localtime(t)) %(
+    #    (t*1000)%1000)
 
 class _FileLogHandler:
     """Helper class for logging.  Represents a file on disk, and allows the
@@ -795,7 +825,7 @@ class _FileLogHandler:
         try:
             parent = os.path.split(self.fname)[0]
             if not os.path.exists(parent):
-                os.mkdirs(parent, 0700)
+                os.makedirs(parent, 0700)
             self.file = open(self.fname, 'a')
         except OSError, e:
             self.file = None
@@ -1124,10 +1154,10 @@ class Duration:
             self.nUnits = seconds
 
     def __str__(self):
-        s = ""
+        plural = ""
         if self.nUnits != 1:
-            s = "s"
-        return "%s %s%s" % (self.nUnits, self.unitName, s)
+            plural = "s"
+        return "%s %s%s" % (self.nUnits, self.unitName, plural)
 
     def __repr__(self):
         return "Duration(%r, %r, %r)" % (self.seconds, self.unitName,
@@ -1460,7 +1490,11 @@ class Lockfile:
         """Acquire this lock.  If we're acquiring the lock for the first time,
            write 'contents' to the lockfile.  If 'blocking' is true, wait
            until we can acquire the lock.  If 'blocking' is false, raise
-           LockfileLocked if we can't acquire the lock."""
+           LockfileLocked if we can't acquire the lock.
+
+           XXXX008 NOTE: This doesn't match the standard lock interface, which
+           defaults to blocking=1.  This should change. XXXX008
+        """
 
         if self.count > 0:
             self.count += 1
@@ -1569,67 +1603,3 @@ def _warn_no_locks():
         _warned_no_locks = 1
         LOG.warn("Mixminion couldn't find a file locking implementation.")
         LOG.warn("  (Simultaneous accesses may lead to data corruption.")
-
-#----------------------------------------------------------------------
-# Threading operations
-
-class ClearableQueue(MessageQueue):
-    """Extended version of python's Queue class that supports removing
-       all the items from the queue."""
-    def clear(self):
-        """Remove all the items from this queue."""
-        # If the queue is empty, return.
-        if not self.esema.acquire(0):
-            return
-        self.mutex.acquire()
-        was_full = self._full()
-        self._clear()
-        assert self._empty()
-        # If the queue used to be full, it isn't anymore.
-        if was_full:
-            self.fsema.release()
-        self.mutex.release()
-
-    def _clear(self):
-        """Backend for _clear"""
-        del self.queue[:]
-
-try:
-    q = MessageQueue()
-    q.put(3)
-    q.get(timeout=10)
-    BUILTIN_QUEUE_HAS_TIMEOUT = 1
-except TypeError:
-    BUILTIN_QUEUE_HAS_TIMEOUT = 0
-del q
-
-if BUILTIN_QUEUE_HAS_TIMEOUT:
-    TimeoutQueue = ClearableQueue
-else:
-    class TimeoutQueue(ClearableQueue):
-        """Helper class for Python 2.2. and earlier: extends the 'get'
-           functionality of Queue.Queue to support a 'timeout' argument.
-           If 'block' is true and timeout is provided, wait for no more
-           than 'timeout' seconds before raising QueueEmpty.
-
-           In Python 2.3 and later, this interface is standard.
-        """
-        def get(self, block=1, timeout=None):
-            if timeout is None or not block:
-                return MessageQueue.get(self, block)
-
-            # This logic is adapted from 'Condition' in the Python
-            # threading module.
-            _time = time.time
-            _sleep = time.sleep
-            deadline = timeout+_time()
-            delay = .0005
-            while 1:
-                try:
-                    return MessageQueue.get(self,0)
-                except QueueEmpty:
-                    remaining = deadline-_time()
-                    if remaining <= 0:
-                        raise
-                    delay = min(delay*2,remaining,0.2)
-                    _sleep(delay)
