@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: Config.py,v 1.72 2003/12/08 02:22:56 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: Config.py,v 1.84 2004/03/23 03:03:36 nickm Exp $
 
 """Configuration file parsers for Mixminion client and server
    configuration.
@@ -61,8 +61,6 @@ try:
 except ImportError:
     pwd = None
 
-from cStringIO import StringIO
-
 import mixminion.Common
 import mixminion.Crypto
 import mixminion.NetUtils
@@ -189,6 +187,14 @@ def _unparseIntervalList(lst):
             result.append(str(d))
     return ", ".join(result)
 
+def _parseList(s):
+    """Validation function.  Parse a comma-separated list of strings."""
+    return [ item.strip() for item in s.split(",") ]
+
+def _parseSeq(s):
+    """Validation function.  Parse a space-separated list of strings."""
+    return [ item.strip() for item in s.split() ]
+
 def _parseInt(integer):
     """Validation function.  Converts a config value to an int.
        Raises ConfigError on failure."""
@@ -221,11 +227,12 @@ def _unparseSize(size):
     names = ["b", "KB", "MB", "GB"]
     idx = 0
     while 1:
-        if (size & 1023) or names[idx] == "GB":
+        if (size & 1023)!=0 or names[idx] == "GB":
             return "%s %s"%(size,names[idx])
         else:
             idx += 1
             size >>= 10
+    raise AssertionError # unreached
 
 def _parseIP(ip):
     """Validation function.  Converts a config value to an IP address.
@@ -296,6 +303,12 @@ def _parseAddressSet_allow(s, allowMode=1):
 def _parseAddressSet_deny(s):
     return _parseAddressSet_allow(s,0)
 
+def _parseEmail(s):
+    s = s.strip()
+    if not mixminion.Common.isSMTPMailbox(s):
+        raise ConfigError("%r is not a valid email address."%s)
+    return s
+
 def _parseCommand(command):
     """Validation function.  Converts a config value to a shell command of
        the form (fname, optionslist). Raises ConfigError on failure."""
@@ -350,7 +363,8 @@ def _parsePublicKey(s):
         raise ConfigError("Invalid exponent on public key")
     return key
 
-# FFFF007/8 stop accepting YYYY/MM/DD
+# FFFF008 stop accepting YYYY/MM/DD.  We've generated the right thing
+# FFFF008 since 0.0.6.
 # Regular expression to match YYYY/MM/DD or YYYY-MM-DD
 _date_re = re.compile(r"^(\d\d\d\d)([/-])(\d\d)([/-])(\d\d)$")
 def _parseDate(s):
@@ -370,7 +384,7 @@ def _parseDate(s):
         raise ConfigError("Invalid date %r"%s)
     return calendar.timegm((yyyy,MM,dd,0,0,0,0,0,0))
 
-# FFFF007/8 stop accepting YYYY/MM/DD
+# FFFF008 stop accepting YYYY/MM/DD
 # Regular expression to match YYYY/MM/DD HH:MM:SS
 _time_re = re.compile(r"^(\d\d\d\d)([/-])(\d\d)([/-])(\d\d)\s+"
                       r"(\d\d):(\d\d):(\d\d)((?:\.\d\d\d)?)$")
@@ -525,6 +539,9 @@ def _readRestrictedConfigFile(contents):
     if fileLines[-1] == '':
         del fileLines[-1]
 
+    if len(fileLines) == 1 and fileLines[0].strip() == '':
+        raise ConfigError("File is empty")
+
     for line in fileLines:
         lineno += 1
         line = line.strip()
@@ -697,6 +714,8 @@ class _ConfigFile:
         "IP" : (_parseIP, str),
         "IP6" : (_parseIP6, str),
         "host" : (_parseHost, str),
+        "list" : (_parseList, ",".join),
+        "seq" : (_parseSeq, " ".join),
         "addressSet_allow" : (_parseAddressSet_allow, str), #XXXX
         "addressSet_deny" : (_parseAddressSet_deny, str), #XXXX
         "command" : (_parseCommand, lambda c,o: " ".join([c," ".join(o)])),
@@ -708,6 +727,7 @@ class _ConfigFile:
         "nickname" : (_parseNickname, str),
         "filename" : (_parseFilename, str),
         "user" : (_parseUser, str),
+        "email" : (_parseEmail, str),
         }
 
     _syntax = None
@@ -735,9 +755,11 @@ class _ConfigFile:
 
         if filename:
             contents = mixminion.Common.readPossiblyGzippedFile(filename)
+            self.fname = filename
             self.__load(contents)
         else:
             assert string is not None
+            self.fname = None
             self.__load(string)
 
     def __load(self, fileContents):
@@ -957,25 +979,30 @@ class ClientConfig(_ConfigFile):
                    'FileParanoia': ('ALLOW', "boolean", "yes"),
                    },
         'DirectoryServers' :
-                   { '__SECTION__' : ('REQUIRE', None, None),
+                   { '__SECTION__' : ('ALLOW', None, None),
                      'ServerURL' : ('ALLOW*', None, None),
                      'MaxSkew' : ('ALLOW', "interval", "10 minutes"),
                      'DirectoryTimeout' : ('ALLOW', "interval", "1 minute") },
         'User' : { 'UserDir' : ('ALLOW', "filename", DEFAULT_USER_DIR) },
-        'Security' : { 'PathLength' : ('ALLOW', "int", "8"),
-                       'SURBAddress' : ('ALLOW', None, None),
-                       'SURBPathLength' : ('ALLOW', "int", "4"),
+        'Security' : { 'SURBAddress' : ('ALLOW', None, None),
                        'SURBLifetime' : ('ALLOW', "interval", "7 days"),
-                       'ForwardPath' : ('ALLOW', None, "*6"),
-                       'ReplyPath' : ('ALLOW', None, "*4"),
-                       'SURBPath' : ('ALLOW', None, "*4"),
+                       'ForwardPath' : ('ALLOW', None, "~5"),
+                       'ReplyPath' : ('ALLOW', None, "~5"),
+                       'SURBPath' : ('ALLOW', None, "~5"),
+                       'BlockServers' : ('ALLOW*', 'list', ""),
+                       'BlockEntries' : ('ALLOW*', 'list', ""),
+                       'BlockExits' : ('ALLOW*', 'list', ""),
+                       #XXXX008; remove these; they've been disabled since 007
+                       'PathLength' : ('ALLOW', None, None),
+                       'SURBPathLength' : ('ALLOW', None, None),
                        },
-        'Network' : { 'ConnectionTimeout' : ('ALLOW', "interval", "2 minutes")}
+        'Network' : { 'ConnectionTimeout' : ('ALLOW', "interval", None),
+                      'Timeout' : ('ALLOW', "interval", None) }
+
         }
     def __init__(self, fname=None, string=None):
         _ConfigFile.__init__(self, fname, string)
 
-
     def prevalidate(self, contents):
         # See if we've been passed a server configuration.
         foundServer = 0
@@ -993,20 +1020,37 @@ class ClientConfig(_ConfigFile):
     def validate(self, lines, contents):
         _validateHostSection(self['Host'])
 
-        security = self['Security']
-        p = security.get('PathLength', 8)
-        if not 0 < p <= 16:
-            raise ConfigError("Path length must be between 1 and 16")
-        if p < 4:
-            LOG.warn("Your default path length is frighteningly low."
-                          "  I'll trust that you know what you're doing.")
-
         t = self['Network'].get('ConnectionTimeout')
-        if t:
-            if int(t) < 5:
-                LOG.warn("Very short connection timeout")
-            elif int(t) > 60:
-                LOG.warn("Very long connection timeout")
+        if t is not None:
+            LOG.warn("The ConnectionTimout option in your .mixminionrc is deprecated; use Timeout instead.")
+        t = self.getTimeout()
+        if int(t) < 5:
+            LOG.warn("Very short network timeout")
+        elif int(t) > 120:
+            LOG.warn("Very long network timeout")
+
+        #XXXX008 safe to remove; has warned since 007rc2
+        security = self.get('Security', {})
+        for deprecatedKey, altKey in [('PathLength', 'ForwardPath'),
+                                      ('SURBPathLength', 'SURBPath')]:
+            if security.get(deprecatedKey,None) is not None:
+                v = security[deprecatedKey]
+                LOG.warn("The %s option in your .mixminionrc is no longer supported; use '%s: *%s' instead",
+                         deprecatedKey, altKey, v)
+
+    def getTimeout(self):
+        """Return the network timeout in this configuration."""
+        network = self.get("Network",{})
+        # The variable is now called 'Timeout'...
+        t = network.get("Timeout",None)
+        if t is not None:
+            return int(t)
+        # ...but older code may call it 'ConnectionTimout'.
+        t = network.get("ConnectionTimeout",None)
+        if t is not None:
+            return int(t)
+        # ...default to 2 minutes.
+        return 120
 
 def _validateHostSection(sec):
     """Helper function: Makes sure that the shared [Host] section is correct;
