@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: Filestore.py,v 1.15 2003/12/04 05:02:50 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: Filestore.py,v 1.17.2.1 2004/04/25 00:25:46 nickm Exp $
 
 """mixminion.Filestore
 
@@ -20,7 +20,6 @@ import dumbdbm
 import errno
 import os
 import stat
-import sys
 import threading
 import time
 import whichdb
@@ -43,12 +42,6 @@ class CorruptedFile(MixError):
 # ======================================================================
 # Filestores.
 
-# Mode to pass to open(2) for creating a new file, and dying if it already
-# exists.
-_NEW_FILE_FLAGS = os.O_WRONLY+os.O_CREAT+os.O_EXCL
-# On windows or (old-school) mac, binary != text.
-_NEW_FILE_FLAGS += getattr(os, 'O_BINARY', 0)
-
 # Any inp_* files older than INPUT_TIMEOUT seconds old are assumed to be
 # trash.
 INPUT_TIMEOUT = 6000
@@ -212,9 +205,10 @@ class BaseStore:
            you're done writing, you must call finishMessage to
            commit your changes, or abortMessage to reject them."""
         while 1:
-            file, handle = getCommonPRNG().openNewFile(self.dir, "inp_", 1,
+            f, handle = getCommonPRNG().openNewFile(self.dir, "inp_", 1,
                                                        "msg_")
-            return file, handle
+            return f, handle
+        raise AssertionError # unreached; appease pychecker
 
     def finishMessage(self, f, handle, _ismeta=0):
         """Given a file and a corresponding handle, closes the file
@@ -437,10 +431,13 @@ class BaseMetadataStore(BaseStore):
 
     def setMetadata(self, handle, object):
         """Change the metadata associated with a given handle."""
+        # On windows or (old-school) mac, binary != text.
+        O_BINARY = getattr(os, 'O_BINARY', 0)
+        flags = os.O_WRONLY|os.O_CREAT|os.O_TRUNC|O_BINARY
         try:
             self._lock.acquire()
             fname = os.path.join(self.dir, "inpm_"+handle)
-            f = os.fdopen(os.open(fname, _NEW_FILE_FLAGS, 0600), "wb")
+            f = os.fdopen(os.open(fname, flags, 0600), "wb")
             cPickle.dump(object, f, 1)
             self.finishMessage(f, handle, _ismeta=1)
             self._metadata_cache[handle] = object
