@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: Fragments.py,v 1.10 2003/11/28 04:14:04 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: Fragments.py,v 1.15 2004/03/23 00:05:32 nickm Exp $
 
 """mixminion.BuildMessage
 
@@ -10,7 +10,7 @@ import math
 import time
 import mixminion._minionlib
 import mixminion.Filestore
-from mixminion.Crypto import ceilDiv, getCommonPRNG, whiten, unwhiten
+from mixminion.Crypto import ceilDiv, getCommonPRNG, sha1, whiten, unwhiten
 from mixminion.Common import disp64, LOG, previousMidnight, MixError, \
      MixFatalError
 from mixminion.Packet import ENC_FWD_OVERHEAD, PAYLOAD_LEN, \
@@ -126,7 +126,7 @@ class FragmentPool:
            old messages in 'dir_db'.
            """
         self.store = mixminion.Filestore.StringMetadataStore(
-            dir,create=1,scrub=1)
+            dir,create=1)
         self.db = FragmentDB(dir+"_db")
         self.rescan()
 
@@ -145,10 +145,11 @@ class FragmentPool:
         del self.store
         del self.states
 
-    def addFragment(self, fragmentPacket, nym=None, now=None):
+    def addFragment(self, fragmentPacket, nym=None, now=None, verbose=0):
         """Given an instance of mixminion.Packet.FragmentPayload, record
            the fragment if appropriate and update the state of the
-           fragment pool if necessary.
+           fragment pool if necessary.  Returns the message ID that was
+           updated, or None if the fragment was redundant or misformed.
 
               fragmentPacket -- the new fragment to add.
               nym -- a string representing the identity that received this
@@ -156,7 +157,13 @@ class FragmentPool:
                   attack where we send 2 fragments to 'MarkTwain' and 2
                   fragments to 'SClemens', and see that the message is
                   reconstructed.]
+              verbose -- if true, log information at the INFO level;
+                  otherwise, log at DEBUG.
         """
+        if verbose:
+            say = LOG.info
+        else:
+            say = LOG.debug
         if now is None:
             now = time.time()
         today = previousMidnight(now)
@@ -165,9 +172,9 @@ class FragmentPool:
         # drop this packet.
         s = self.db.getStatusAndTime(fragmentPacket.msgID)
         if s:
-            LOG.debug("Dropping fragment of %s message %r",
-                      s[0].lower(), disp64(fragmentPacket.msgID,13))
-            return
+            say("Dropping fragment of %s message %r",
+                s[0].lower(), disp64(fragmentPacket.msgID,12))
+            return None
 
         # Otherwise, create a new metadata object for this fragment...
         meta = FragmentMetadata(messageid=fragmentPacket.msgID,
@@ -177,7 +184,8 @@ class FragmentPool:
                                  chunkNum=None,
                                  overhead=fragmentPacket.getOverhead(),
                                  insertedDate=today,
-                                 nym=nym)
+                                 nym=nym,
+                                 digest=sha1(fragmentPacket.data))
         # ... and allocate or find the MessageState for this message.
         state = self._getState(meta)
         try:
@@ -188,17 +196,21 @@ class FragmentPool:
             h = self.store.queueMessageAndMetadata(fragmentPacket.data, meta)
             # And *now* update the message state.
             state.addFragment(h, meta)
-            LOG.debug("Stored fragment %s of message %s",
-                      fragmentPacket.index, disp64(fragmentPacket.msgID,13))
-        except MismatchedFragment:
+            say("Stored fragment %s of message %s",
+                fragmentPacket.index+1, disp64(fragmentPacket.msgID,12))
+            return fragmentPacket.msgID
+        except MismatchedFragment, s:
             # Remove the other fragments, mark msgid as bad.
-            LOG.warn("Found inconsistent fragment %s in message %s",
-                      fragmentPacket.index, disp64(fragmentPacket.msgID,13))
+            LOG.warn("Found inconsistent fragment %s in message %s: %s",
+                     fragmentPacket.index+1, disp64(fragmentPacket.msgID,12),
+                     s)
             self._deleteMessageIDs({ meta.messageid : 1}, "REJECTED", now)
+            return None
         except UnneededFragment:
             # Discard this fragment; we don't need it.
-            LOG.debug("Dropping unneeded fragment %s of message %s",
-                      fragmentPacket.index, disp64(fragmentPacket.msgID,13))
+            say("Dropping unneeded fragment %s of message %s",
+                fragmentPacket.index+1, disp64(fragmentPacket.msgID,12))
+            return None
 
     def getReadyMessage(self, msgid):
         """Return the complete message associated with messageid 'msgid'.
@@ -310,9 +322,9 @@ class FragmentPool:
            undeliverable.
 
               messageIDSet -- a map from 20-byte messageID to 1.
-              why -- 'REJECTED' or 'COMPLETED'.
+              why -- 'REJECTED' or 'COMPLETED' or '?'
         """
-        assert why in ("REJECTED", "COMPLETED")
+        assert why in ("REJECTED", "COMPLETED", "?")
         if not messageIDSet:
             return
         if today is None:
@@ -321,11 +333,23 @@ class FragmentPool:
         if why == 'REJECTED':
             LOG.debug("Removing bogus messages by IDs: %s",
                       messageIDSet.keys())
-        else:
+        elif why == "COMPLETED":
             LOG.debug("Removing completed messages by IDs: %s",
                       messageIDSet.keys())
+        else:
+            LOG.debug("Removing messages by IDs: %s",
+                      messageIDSet.keys())
+
         for mid in messageIDSet.keys():
-            self.db.markStatus(mid, why, today)
+            if why == "?":
+                state = self.states[mid]
+                if state.isDone:
+                    whythis = "COMPLETED"
+                else:
+                    whythis = "REJECTED"
+            else:
+                whythis = why
+            self.db.markStatus(mid, whythis, today)
             try:
                 del self.states[mid]
             except KeyError:
@@ -348,6 +372,39 @@ class FragmentPool:
             self.states[fm.messageid] = state
             return state
 
+    def getStateByMsgID(self, msgid):
+        """Given a message ID (either a 20-byte full ID or a 12-byte
+           pretty-printed ID prefix), return a MessageState object for
+           the corresponding message, or None if the message is not
+           recognized."""
+        if len(msgid) == 20:
+            return self.state.get(msgid,None)
+        elif len(msgid) == 12:
+            target = binascii.a2b_base64(msgid)
+            for i in self.states.keys():
+                if i.startswith(target):
+                    return self.states[i]
+        return None
+
+    def listMessages(self):
+        """Return a map from pretty-printed message ID to dicts mapping:
+               'size' to the size of the message, in bytes
+               'nym' to the pseudonym receiving the message
+               'have' to the number of packets we have so far
+               'need' to the number of additional packets we need.
+        """
+        result = {}
+        for msgid in self.states.keys():
+            state = self.states[msgid]
+            have, need = state.getCompleteness()
+            result[disp64(msgid,12)] = {
+                'size' : state.params.length,
+                'nym' : state.nym,
+                'have' : have,
+                'need' : need
+                }
+        return result
+
 # ======================================================================
 
 class MismatchedFragment(Exception):
@@ -377,8 +434,9 @@ class FragmentMetadata:
     #    ENC_FWD_OVERHEAD.
     # insertedDate -- Midnight GMT before the day this fragment was received.
     # nym -- name of the identity that received this fragment.
+    # digest -- digest of the fragment/chunk; None for pre-0.0.7
     def __init__(self, messageid, idx, size, isChunk, chunkNum, overhead,
-                 insertedDate, nym):
+                 insertedDate, nym, digest):
         self.messageid = messageid
         self.idx = idx
         self.size = size
@@ -387,17 +445,23 @@ class FragmentMetadata:
         self.overhead = overhead
         self.insertedDate = insertedDate
         self.nym = nym
+        self.digest = digest
 
     def __getstate__(self):
-        return ("V0", self.messageid, self.idx, self.size,
+        return ("V1", self.messageid, self.idx, self.size,
                 self.isChunk, self.chunkNum, self.overhead, self.insertedDate,
-                self.nym)
+                self.nym, self.digest)
 
     def __setstate__(self, state):
         if state[0] == 'V0':
             (_, self.messageid, self.idx, self.size,
              self.isChunk, self.chunkNum, self.overhead, self.insertedDate,
              self.nym) = state
+            self.digest = None
+        elif state[0] == 'V1':
+            (_, self.messageid, self.idx, self.size,
+             self.isChunk, self.chunkNum, self.overhead, self.insertedDate,
+             self.nym,self.digest) = state
         else:
             raise MixFatalError("Unrecognized fragment state")
 
@@ -418,7 +482,7 @@ class MessageState:
     # params -- an instance of FragmentationParams for this message.
     # chunks -- a map from chunk number to tuples of (handle within the pool,
     #     FragmentMetadata object).  For completed chunks.
-    # fragmentsByChunk -- a map from chunk number to maps from
+    # fragmentsByChunk -- a list mapping chunk number to maps from
     #     index-within-chunk to (handle,FragmentMetadata)
     # readyChunks -- a map whose keys are the numbers of chunks that
     #     are ready for reconstruction, but haven't been reconstructed
@@ -451,19 +515,30 @@ class MessageState:
         assert self.isDone()
         return [ self.chunks[i][0] for i in xrange(self.params.nChunks) ]
 
+    def getCompleteness(self):
+        """Return a tuple of (have,need), where 'need' is the final number
+           of packets needed to reconstruct the message, and 'have' is the
+           number we have so far."""
+        need = self.params.k * self.params.nChunks
+        have = self.params.k * len(self.chunks)
+        for d in self.fragmentsByChunk:
+            have += min(len(d),self.params.k)
+        return have, need
+
     def addChunk(self, h, fm):
         """Register a chunk with handle h and FragmentMetadata fm.  If the
            chunk is inconsistent with other fragments of this message,
            raise MismatchedFragment."""
         assert fm.isChunk
         assert fm.messageid == self.messageid
-        if (fm.size != self.params.length or
-            fm.overhead != self.overhead or
-            self.chunks.has_key(fm.chunkNum)):
-            raise MismatchedFragment
-
+        if fm.size != self.params.length:
+            raise MismatchedFragment("Mismatched message length")
+        if fm.overhead != self.overhead:
+            raise MismatchedFragment("Mismatched packet overhead")
+        if self.chunks.has_key(fm.chunkNum):
+            raise MismatchedFragment("Duplicate chunks")
         if fm.nym != self.nym:
-            raise MismatchedFragment
+            raise MismatchedFragment("Fragments received for differing identities")
 
         if self.inserted > fm.insertedDate:
             self.inserted = fm.insertedDate
@@ -485,9 +560,12 @@ class MessageState:
            fragment--just raise exceptions as needed."""
         assert fm.messageid == self.messageid
 
-        if (fm.size != self.params.length or
-            fm.overhead != self.overhead):
-            raise MismatchedFragment
+        if fm.size != self.params.length:
+            raise MismatchedFragment("mismatched message size")
+        if fm.overhead != self.overhead:
+            raise MismatchedFragment("mismatched fragment payload size")
+        if fm.nym != self.nym:
+            raise MismatchedFragment("mismatched identities")
 
         chunkNum, pos = self.params.getPosition(fm.idx)
         if chunkNum >= self.params.nChunks:
@@ -498,7 +576,11 @@ class MessageState:
             raise UnneededFragment
 
         if self.fragmentsByChunk[chunkNum].has_key(pos):
-            raise MismatchedFragment
+            previous = self.fragmentsByChunk[chunkNum][pos][1]
+            if previous.digest is None or previous.digest == fm.digest:
+                raise UnneededFragment("already seen this fragment")
+            else:
+                raise MismatchedFragment("multiple fragments for one position")
 
         if noop:
             return
@@ -533,7 +615,8 @@ class MessageState:
                                    idx=chunkno, size=self.params.length,
                                    isChunk=1, chunkNum=chunkno,
                                    overhead=self.overhead,
-                                   insertedDate=minDate, nym=self.nym)
+                                   insertedDate=minDate, nym=self.nym,
+                                   digest=sha1(chunkText))
             # Queue the chunk.
             h2 = store.queueMessageAndMetadata(chunkText, fm2)
             del chunkText
