@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: Packet.py,v 1.69 2003/12/14 01:29:25 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: Packet.py,v 1.75 2004/03/06 00:04:38 nickm Exp $
 """mixminion.Packet
 
    Functions, classes, and constants to parse and unparse Mixminion
@@ -48,6 +48,7 @@ if sys.version_info[:3] < (2,2,0):
 
 # Major and minor number for the understood packet format.
 MAJOR_NO, MINOR_NO = 0,3
+PACKET_VERSION = "%s.%s"%(MAJOR_NO,MINOR_NO)
 
 # Length of a Mixminion packet
 PACKET_LEN = 1 << 15
@@ -93,12 +94,6 @@ NEWS_TYPE      = 0x0102  # Post the message to some ngs, and maybe mail it too
 FRAGMENT_TYPE  = 0x0103  # Find the actual delivery info in the message payload
 MAX_EXIT_TYPE  = 0xFFFF
 
-# Set of exit types that don't get tag fields.
-# XXXX007 This interface is really brittle; it needs to change.  I added it
-# XXXX007 in order to allow 'fragment' to be an exit type without adding a
-# XXXX007 needless tag field to every fragment routing info.
-_TYPES_WITHOUT_TAGS = { FRAGMENT_TYPE : 1 }
-
 def typeIsSwap(tp):
     return tp in (SWAP_FWD_IPV4_TYPE,SWAP_FWD_HOST_TYPE)
 
@@ -161,8 +156,8 @@ def parseSubheader(s):
     underflow = ""
     if rlen < len(ri):
         ri, underflow = ri[:rlen], ri[rlen:]
-    if rt >= MIN_EXIT_TYPE and not _TYPES_WITHOUT_TAGS.get(rt) and rlen < TAG_LEN:
-        raise ParseError("Subheader missing tag")
+##     if rt >= MIN_EXIT_TYPE and not _TYPES_WITHOUT_TAGS.get(rt) and rlen < TAG_LEN:
+##         raise ParseError("Subheader missing tag")
     return Subheader(major,minor,secret,digest,rt,ri,rlen,underflow)
 
 class Subheader:
@@ -201,27 +196,27 @@ class Subheader:
                 "routingtype=%(routingtype)r, routinginfo=%(routinginfo)r, "+
                 "routinglen=%(routinglen)r)")% self.__dict__
 
-    def getExitAddress(self):
+    def getExitAddress(self, tagged=1):
         """Return the part of the routingInfo that contains the delivery
-           address.  (Requires that routingType is an exit type.)"""
+           address.  (Requires that routingType is an exit type.)  If
+           'tagged' is true, ignore the first TAG_LEN bytes of the
+           routingInfo.
+        """
         assert self.routingtype >= MIN_EXIT_TYPE
-        #XXXX007 This interface is completely insane.  Change it.
-        if _TYPES_WITHOUT_TAGS.get(self.routingtype):
+        if not tagged:
             return self.routinginfo
         else:
-            assert len(self.routinginfo) >= TAG_LEN
+            if len(self.routinginfo) < TAG_LEN:
+                raise ParseError("Missing tag")
             return self.routinginfo[TAG_LEN:]
 
     def getTag(self):
         """Return the part of the routingInfo that contains the decoding
            tag. (Requires that routingType is an exit type.)"""
         assert self.routingtype >= MIN_EXIT_TYPE
-        #XXXX007 This interface is completely insane.  Change it.
-        if _TYPES_WITHOUT_TAGS.get(self.routingtype):
-            return ""
-        else:
-            assert len(self.routinginfo) >= TAG_LEN
-            return self.routinginfo[:TAG_LEN]
+        if len(self.routinginfo) < TAG_LEN:
+            raise ParseError("Missing tag")
+        return self.routinginfo[:TAG_LEN]
 
     def setRoutingInfo(self, info):
         """Change the routinginfo, and the routinglength to correspond."""
@@ -291,22 +286,22 @@ def parsePayload(payload):
     bit0 = ord(payload[0]) & 0x80
     if bit0:
         # We have a fragment
-        idxhi, idxlo, hash, msgID, msgLen = \
+        idxhi, idxlo, digest, msgID, msgLen = \
                struct.unpack(FRAGMENT_UNPACK_PATTERN,
                              payload[:FRAGMENT_PAYLOAD_OVERHEAD])
         idx = ((idxhi & 0x7f) << 16) + idxlo
         contents = payload[FRAGMENT_PAYLOAD_OVERHEAD:]
         if msgLen <= len(contents):
             raise ParseError("Payload has an invalid size field")
-        return FragmentPayload(idx,hash,msgID,msgLen,contents)
+        return FragmentPayload(idx,digest,msgID,msgLen,contents)
     else:
         # We have a singleton
-        size, hash = struct.unpack(SINGLETON_UNPACK_PATTERN,
+        size, digest = struct.unpack(SINGLETON_UNPACK_PATTERN,
                                    payload[:SINGLETON_PAYLOAD_OVERHEAD])
         contents = payload[SINGLETON_PAYLOAD_OVERHEAD:]
         if size > len(contents):
             raise ParseError("Payload has invalid size field")
-        return SingletonPayload(size,hash,contents)
+        return SingletonPayload(size,digest,contents)
 
 # A singleton payload starts with a 0 bit, 15 bits of size, and a 20-byte hash
 SINGLETON_UNPACK_PATTERN = "!H%ds" % (DIGEST_LEN)
@@ -321,9 +316,9 @@ class _Payload:
 class SingletonPayload(_Payload):
     """Represents the payload for a standalone mixminion message.
        Fields:  size, hash, data.  (Note that data is padded.)"""
-    def __init__(self, size, hash, data):
+    def __init__(self, size, digest, data):
         self.size = size
-        self.hash = hash
+        self.hash = digest
         self.data = data
 
     def computeHash(self):
@@ -364,9 +359,9 @@ class SingletonPayload(_Payload):
 class FragmentPayload(_Payload):
     """Represents the fields of a decoded fragment payload.
     """
-    def __init__(self, index, hash, msgID, msgLen, data):
+    def __init__(self, index, digest, msgID, msgLen, data):
         self.index = index
-        self.hash = hash
+        self.hash = digest
         self.msgID = msgID
         self.msgLen = msgLen
         self.data = data
@@ -529,8 +524,8 @@ class ReplyBlock:
         self.encryptionKey = key
 
     def format(self):
-        from mixminion.ServerInfo import displayServer
-        hash = binascii.b2a_hex(sha1(self.pack()))
+        import mixminion.ServerInfo
+        digest = binascii.b2a_hex(sha1(self.pack()))
         expiry = formatTime(self.timestamp)
         if self.routingType == SWAP_FWD_IPV4_TYPE:
             routing = parseIPV4Info(self.routingInfo)
@@ -538,9 +533,10 @@ class ReplyBlock:
             routing = parseMMTPHostInfo(self.routingInfo)
         else:
             routing = None
+        server = mixminion.ServerInfo.displayServerByRouting(routing)
         return """Reply block hash: %s
 Expires at: %s GMT
-First server is: %s""" % (hash, expiry, displayServer(routing))
+First server is: %s""" % (digest, expiry, server)
 
     def pack(self):
         """Returns the external representation of this reply block"""
@@ -597,7 +593,7 @@ class IPV4Info:
 
        Fields: ip (a dotted quad string), port (an int from 0..65535),
        and keyinfo (a digest)."""
-    #XXXX007/8 phase this out.
+    #XXXX008 phase this out: we have supported hostnames since 0.0.6.
     def __init__(self, ip, port, keyinfo):
         """Construct a new IPV4Info"""
         assert 0 <= port <= 65535
@@ -903,6 +899,8 @@ def parseMessageAndHeaders(message):
             LOG.warn("Could not parse headers on message; not using them.")
             return message, headers
 
+    raise AssertionError # Unreached; appease pychecker
+
 #----------------------------------------------------------------------
 # COMPRESSION FOR PAYLOADS
 
@@ -1000,7 +998,7 @@ def _validateZlib():
 
     _ZLIB_LIBRARY_OK = 0.5
     if ver in ("1.1.2", "1.1.3", "1.1.4", "1.2.0", "1.2.0.1", "1.2.0.2",
-               "1.2.0.3", "1.2.0.4", "1.2.0.5", "1.2.0.6", "1.2.0.7", 
+               "1.2.0.3", "1.2.0.4", "1.2.0.5", "1.2.0.6", "1.2.0.7",
                "1.2.0.8", "1.2.1"):
         _ZLIB_LIBRARY_OK = 1
         return
