@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: ServerInfo.py,v 1.69 2003/12/12 22:56:16 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: ServerInfo.py,v 1.83.2.1 2004/04/27 06:00:47 nickm Exp $
 
 """mixminion.ServerInfo
 
@@ -8,22 +8,20 @@
    descriptors.
    """
 
-__all__ = [ 'ServerInfo', 'ServerDirectory', 'displayServer' ]
+__all__ = [ 'ServerInfo', 'ServerDirectory', 'displayServerByRouting' ]
 
 import re
 import time
-import types
 
 import mixminion.Config
-import mixminion.Crypto
 import mixminion.MMTPClient
 import mixminion.Packet
 
 from mixminion.Common import IntervalSet, LOG, MixError, createPrivateDir, \
-    formatBase64, formatDate, formatTime, readPossiblyGzippedFile
+     formatBase64, formatDate, formatTime, readPossiblyGzippedFile
 from mixminion.Config import ConfigError
-from mixminion.Packet import IPV4Info, MMTPHostInfo
-from mixminion.Crypto import CryptoError, DIGEST_LEN, pk_check_signature
+from mixminion.Crypto import CryptoError, DIGEST_LEN, pk_check_signature, \
+     pk_encode_public_key, pk_sign, sha1
 
 # Longest allowed Contact email
 MAX_CONTACT = 256
@@ -41,55 +39,65 @@ PACKET_KEY_BYTES = 2048 >> 3
 MMTP_KEY_BYTES = 1024 >> 3
 
 # ----------------------------------------------------------------------
-def displayServer(s):
-    """Return the best possible human-readable name for a server 's'.
-       's' must be one of: None, IPV4Info, MMTPHostInfo, ServerInfo,
-       string.
+def displayServerByRouting(s):
+    """Return the best possible human-readable name for a server with
+       routinginfo 's'.  's' must be one of: IPV4Info, MMTPHostInfo.
     """
-    if isinstance(s, types.StringType):
-        return s
-    elif isinstance(s, ServerInfo):
-        if s.getHostname():
-            addr = "%s:%s" % (s.getHostname(), s.getPort())
-        else:
-            addr = "%s:%s" % (s.getIP(), s.getPort())
-        nickname = "'%s'" % s.getNickname()
-    elif (isinstance(s, mixminion.Packet.IPV4Info) or
-          isinstance(s, mixminion.Packet.MMTPHostInfo)):
-        nickname = getNicknameByKeyID(s.keyinfo)
-        if nickname:
-            nickname = "'%s'" % nickname
-        else:
-            nickname = 'server'
-        if isinstance(s, mixminion.Packet.IPV4Info):
-            addr = "%s:%s" % (s.ip, s.port)
-        else:
-            addr = "%s:%s" % (s.hostname, s.port)
-    elif s is None:
-        return "unknown server"
+    assert (isinstance(s, mixminion.Packet.IPV4Info) or
+            isinstance(s, mixminion.Packet.MMTPHostInfo))
+    nickname = getNicknameByKeyID(s.keyinfo)
+    if isinstance(s, mixminion.Packet.IPV4Info):
+        addr = "%s:%s" % (s.ip, s.port)
     else:
-        assert 0
-
+        addr = "%s:%s" % (s.hostname, s.port)
+    if nickname:
+        nickname = "'%s'"%nickname
+    else:
+        nickname = "server"
     return "%s at %s" % (nickname, addr)
 
+def displayServerByAddress(ip, port, hostname=None):
+    """Return the best human-reasable name for a server at IP 'ip', hostname
+       'hostname', connecting from port 'port'.  Note that because reverse
+       caching is unreliable, and IP-based authentication is a lie, any
+       inferred nicknames are displayed as questionable.
+    """
+    nickname = None
+    if hostname is not None:
+        nickname = getNicknameByAddress(hostname)
+    if nickname is None:
+        nickname = getNicknameByAddress(ip)
+    if nickname is None:
+        nickname = ""
+    else:
+        nickname = " (%s?)" % nickname
+    return "host at %s:%s%s" %(ip, port, nickname)
+
 def getNicknameByKeyID(keyid):
     """Given a 20-byte keyid, look up the nickname of the corresponding
-       server.  Return the nickname on success and None if we don't recognize
-       the server.
-
-       FFFF Right now, this is not supported for servers, since they don't
-       FFFF download directories.
+       server.  Return the nickname on success and None if we don't
+       recognize the server.  If multiple servers match, return their
+       nicknames separated by slashes.
     """
     #FFFF Be cleverer with all-zero keyids.
-    if _keyIDToNicknameFn:
+    if _keyIDToNicknameFn is not None:
         return _keyIDToNicknameFn(keyid)
     else:
         return None
 
-# This variable should hold None, or a function that maps from keyids to
-# nicknames, and returns None on failure.  Currently set by
-# ClientDirectory.ClientDirectory._installAsKeyIDResolver().
+def getNicknameByAddress(address):
+    """Given an IP or hostname, look up the nickname of the corresponding
+       server.  Returns as for getNicknameByKeyID.
+    """
+    if _addressToNicknameFn is not None:
+        return _addressToNicknameFn(address)
+    else:
+        return None
+
+# These variables should hold None, or functions that implement getNicknameBy*.
+# Currently set by ClientDirectory.ClientDirectory._installAsKeyIDResolver().
 _keyIDToNicknameFn = None
+_addressToNicknameFn = None
 
 # ----------------------------------------------------------------------
 
@@ -114,43 +122,46 @@ class ServerInfo(mixminion.Config._ConfigFile):
                      "Published": ("REQUIRE", "time", None),
                      "Valid-After": ("REQUIRE", "date", None),
                      "Valid-Until": ("REQUIRE", "date", None),
+                     #XXXX008 change this to 'require': servers have all
+                     #XXXX008 had it since 007.
                      "Contact": ("ALLOW", None, None),
                      "Comments": ("ALLOW", None, None),
                      "Packet-Key": ("REQUIRE", "publicKey", None),
                      "Contact-Fingerprint": ("ALLOW", None, None),
-                     "Packet-Formats": ("ALLOW", None, None),#XXXX007 remove
-                     # XXXX010 change these next few to "REQUIRE".
-                     "Packet-Versions": ("ALLOW", None, None),
+                     "Packet-Formats": ("ALLOW", None, None),#XXXX008 remove
+                     # XXXX008 change these next few to "REQUIRE"; servers
+                     # XXXX008 have had them all since 0.0.5
+                     "Packet-Versions": ("ALLOW", "list", '0.3'),
                      "Software": ("ALLOW", None, None),
                      "Secure-Configuration": ("ALLOW", "boolean", None),
                      "Why-Insecure": ("ALLOW", None, None),
                      },
         "Incoming/MMTP" : {
                      "Version": ("REQUIRE", None, None),
-                     "IP": ("ALLOW", "IP", None),#XXXX007/8 remove
-                     "Hostname": ("ALLOW", "host", None),#XXXX008 require
+                     "IP": ("ALLOW", "IP", None),#XXXX008 remove
+                     "Hostname": ("ALLOW", "host", None),#XXXX008 require;since 0.0.6
                      "Port": ("REQUIRE", "int", None),
-                     "Key-Digest": ("ALLOW", "base64", None),#XXXX007/8 rmv
-                     "Protocols": ("REQUIRE", None, None),
+                     "Key-Digest": ("ALLOW", "base64", None),#XXXX008 rmv; not used since 0.0.5
+                     "Protocols": ("REQUIRE", "list", None),
                      "Allow": ("ALLOW*", "addressSet_allow", None),
                      "Deny": ("ALLOW*", "addressSet_deny", None),
                      },
         "Outgoing/MMTP" : {
                      "Version": ("REQUIRE", None, None),
-                     "Protocols": ("REQUIRE", None, None),
+                     "Protocols": ("REQUIRE", "list", None),
                      "Allow": ("ALLOW*", "addressSet_allow", None),
                      "Deny": ("ALLOW*", "addressSet_deny", None),
                      },
         "Delivery/MBOX" : {
                      "Version": ("REQUIRE", None, None),
-                     # XXXX007 change to 'REQUIRE'
+                     # XXXX008 change to 'REQUIRE'; since 0.0.6
                      "Maximum-Size": ("ALLOW", "int", "32"),
-                     # XXXX007 change to 'REQUIRE'
+                     # XXXX008 change to 'REQUIRE'; since 0.0.6
                      "Allow-From": ("ALLOW", "boolean", "yes"),
                      },
         "Delivery/SMTP" : {
                      "Version": ("REQUIRE", None, None),
-                     # XXXX007 change to 'REQUIRE'
+                     # XXXX008 change to 'REQUIRE'; since 0.0.6
                      "Maximum-Size": ("ALLOW", "int", "32"),
                      "Allow-From": ("ALLOW", "boolean", "yes"),
                      },
@@ -311,10 +322,6 @@ class ServerInfo(mixminion.Config._ConfigFile):
     def getHostname(self):
         """Return this server's Hostname. (Returns None for servers running
            version 0.0.5 or earlier.)"""
-        #XXXX006 remove this.  0.0.6alpha1 could crash when it got hostnames.
-        #XXXX006 Sadly, some people installed it anyway.
-        if self['Server'].get("Software","").endswith("0.0.6alpha1"):
-            return None
         return self['Incoming/MMTP'].get("Hostname")
 
     def getPort(self):
@@ -326,9 +333,8 @@ class ServerInfo(mixminion.Config._ConfigFile):
         return self['Server']['Packet-Key']
 
     def getKeyDigest(self):
-        """Returns a hash of this server's MMTP key"""
-        return mixminion.Crypto.sha1(
-            mixminion.Crypto.pk_encode_public_key(self['Server']['Identity']))
+        """Returns a hash of this server's identity key."""
+        return sha1(pk_encode_public_key(self['Server']['Identity']))
 
     def getIPV4Info(self):
         """Returns a mixminion.Packet.IPV4Info object for routing messages
@@ -336,7 +342,7 @@ class ServerInfo(mixminion.Config._ConfigFile):
            or earlier.)"""
         ip = self.getIP()
         if ip is None: return None
-        return IPV4Info(ip, self.getPort(), self.getKeyDigest())
+        return mixminion.Packet.IPV4Info(ip, self.getPort(), self.getKeyDigest())
 
     def getMMTPHostInfo(self):
         """Returns a mixminion.Packet.MMTPHostInfo object for routing messages
@@ -344,7 +350,8 @@ class ServerInfo(mixminion.Config._ConfigFile):
            or later.)"""
         host = self.getHostname()
         if host is None: return None
-        return MMTPHostInfo(host, self.getPort(), self.getKeyDigest())
+        return mixminion.Packet.MMTPHostInfo(
+            host, self.getPort(), self.getKeyDigest())
 
     def getRoutingInfo(self):
         """Return whichever of MMTPHostInfo or IPV4 info is best for
@@ -365,15 +372,22 @@ class ServerInfo(mixminion.Config._ConfigFile):
         inc = self['Incoming/MMTP']
         if not inc.get("Version"):
             return []
-        return [ s.strip() for s in inc["Protocols"].split(",") ]
+        return inc["Protocols"]
 
     def getOutgoingMMTPProtocols(self):
         """Return a list of the MMTP versions supported by this this server
            for outgoing packets."""
-        inc = self['Outgoing/MMTP']
-        if not inc.get("Version"):
+        out = self['Outgoing/MMTP']
+        if not out.get("Version"):
             return []
-        return [ s.strip() for s in inc["Protocols"].split(",") ]
+        return out["Protocols"]
+
+    def supportsPacketVersion(self):
+        """Return true iff we can build packets in a format this server
+           recognizes."""
+        formats = self['Server'].get("Packet-Versions")
+        if formats == None: formats = [ "0.3" ]
+        return mixminion.Packet.PACKET_VERSION in formats
 
     def canRelayTo(self, otherDesc):
         """Return true iff this server can relay packets to the server
@@ -382,16 +396,22 @@ class ServerInfo(mixminion.Config._ConfigFile):
             return 1
         myOutProtocols = self.getOutgoingMMTPProtocols()
         otherInProtocols = otherDesc.getIncomingMMTPProtocols()
+        if not self.getHostname() and not other.getIP():
+            return 0
         for out in myOutProtocols:
             if out in otherInProtocols:
                 return 1
         return 0
 
+    def canRelay(self):
+        """Return 1"""
+        return 1
+
     def canStartAt(self):
         """Return true iff this server is one we (that is, this
            version of Mixminion) can send packets to directly."""
         myInProtocols = self.getIncomingMMTPProtocols()
-        for out in mixminion.MMTPClient.BlockingClientConnection.PROTOCOL_VERSIONS:
+        for out in mixminion.MMTPClient.MMTPClientConnection.PROTOCOL_VERSIONS:
             if out in myInProtocols:
                 return 1
         return 0
@@ -553,14 +573,15 @@ class ServerDirectory:
         servercontents = [ "[Server]\n%s"%s for s in sections[1:] ]
 
         self.header = _DirectoryHeader(headercontents, digest)
-        self.goodServerNames = [name.strip().lower() for name in
-                   self.header['Directory']['Recommended-Servers'].split(",")]
+        self.goodServerNames = [name.lower() for name in
+                   self.header['Directory']['Recommended-Servers'] ]
         servers = [ ServerInfo(string=s,
                                validatedDigests=validatedDigests)
                     for s in servercontents ]
         self.allServers = servers[:]
-        self.servers = [ s for s in servers
-                         if s.getNickname().lower() in self.goodServerNames ]
+        goodServers = [ s for s in servers
+                        if s.getNickname().lower() in self.goodServerNames ]
+        self.servers = goodServers
 
     def getServers(self):
         """Return a list of recommended ServerInfo objects in this directory"""
@@ -597,7 +618,7 @@ class _DirectoryHeader(mixminion.Config._ConfigFile):
                        "Published": ("REQUIRE", "time", None),
                        "Valid-After": ("REQUIRE", "date", None),
                        "Valid-Until": ("REQUIRE", "date", None),
-                       "Recommended-Servers": ("REQUIRE", None, None),
+                       "Recommended-Servers": ("REQUIRE", "list", None),
                        },
         'Signature': {"__SECTION__": ("REQUIRE", None, None),
                  "DirectoryIdentity": ("REQUIRE", "publicKey", None),
@@ -605,8 +626,8 @@ class _DirectoryHeader(mixminion.Config._ConfigFile):
                  "DirectorySignature": ("REQUIRE", "base64", None),
                       },
         'Recommended-Software': {"__SECTION__": ("ALLOW", None, None),
-                "MixminionClient": ("ALLOW", None, None),
-                "MixminionServer": ("ALLOW", None, None), }
+                "MixminionClient": ("ALLOW", "seq", None),
+                "MixminionServer": ("ALLOW", "seq", None), }
         }
     def __init__(self, contents, expectedDigest):
         """Parse a directory header out of a provided string; validate it
@@ -696,12 +717,12 @@ def _getDigestImpl(info, regex, digestField=None, sigField=None, rsa=None):
         s = m.group(0)
         return s[:s.index(':')+1]
     info = regex.sub(replaceFn, info, 2)
-    digest = mixminion.Crypto.sha1(info)
+    digest = sha1(info)
 
     if rsa is None:
         return digest
 
-    signature = mixminion.Crypto.pk_sign(digest,rsa)
+    signature = pk_sign(digest,rsa)
     digest = formatBase64(digest)
     signature = formatBase64(signature)
     def replaceFn2(s, digest=digest, signature=signature,
