@@ -1,5 +1,5 @@
 # Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: DirMain.py,v 1.16 2003/06/06 06:04:58 nickm Exp $
+# $Id: DirMain.py,v 1.20 2003/11/28 04:14:04 nickm Exp $
 
 """mixminion.directory.DirMain
 
@@ -28,6 +28,7 @@ Usage: mixminion dir <command>
       update                   [Process updates for currently known servers]
       generate                 [Generate and sign a new directory]
       fingerprint              [Return the fingerprint of this directory's pk]
+      rebuildcache             [Rebuild a corrupted or removed identity cache]
 """.strip()
 
 def getDirectory():
@@ -63,7 +64,7 @@ def cmd_init(args):
     """[Entry point] Set up a new set of directory files."""
     if args:
         raise UIError("mixminion dir initialize takes no arguments")
-    
+
     d = getDirectory()
     d.setupDirectories()
     d.getServerList()
@@ -75,7 +76,7 @@ def cmd_update(args):
        as part of a cron job."""
     if args:
         raise UIError("mixminion dir update takes no arguments")
-    
+
     d = getDirectory()
     serverList = d.getServerList()
     inbox = d.getInbox()
@@ -90,6 +91,17 @@ def cmd_list(args):
     inbox = d.getInbox()
     inbox.listNewPendingServers(sys.stdout)
 
+def cmd_rebuildcache(args):
+    """[Entry point] Reconstruct the ID cache from the contents of the
+       'servers' directory.
+    """
+    if args:
+        raise UIError("mixminion dir rebuildcache takes no arguments")
+    d = getDirectory()
+    serverList = d.getServerList()
+    serverList.rebuildIDCache()
+    d.getIDCache().save()
+
 def cmd_import(args):
     """[Entry point] Import descriptors for new servers, by nickname."""
     d = getDirectory()
@@ -125,7 +137,7 @@ def cmd_generate(args):
 
     config = d.getConfig()
 
-    badServers = config['Directory'].get('BadServer', [])
+    badServers = config['Directory'].get('BadServer', [])[:]
     badServerFiles = config['Directory'].get('BadServerFile', [])
     for fn in badServerFiles:
         if not os.path.exists(fn):
@@ -138,17 +150,21 @@ def cmd_generate(args):
                 badServers.append(ln)
         f.close()
 
+    excludeServers = config['Directory'].get("ExcludeServer",[])[:]
+    excludeServers = [ nn.strip().lower() for nn in excludeServers ]
+
     location = config['Publishing']['Location']
     print "(Bad servers==%r)"%badServers
 
     now = time.time()
     tomorrow = now+60*60*24
     twoWeeks = 60*60*24*14
-    
+
     serverList.generateDirectory(startAt=now, endAt=tomorrow,
                                  extraTime=twoWeeks,
                                  identityKey=key,
-                                 badServers=badServers)
+                                 badServers=badServers,
+                                 excludeServers=excludeServers)
     print "Directory generated; publishing."
 
     fname = serverList.getDirectoryFilename()
@@ -178,7 +194,8 @@ SUBCOMMANDS = { 'initialize' : cmd_init,
                 'list' : cmd_list,
                 'import-new' : cmd_import,
                 'generate' : cmd_generate,
-                'fingerprint' : cmd_fingerprint
+                'fingerprint' : cmd_fingerprint,
+                'rebuildcache' : cmd_rebuildcache
                 }
 
 def main(cmd, args):
