@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: Modules.py,v 1.67 2003/12/14 01:43:25 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: Modules.py,v 1.73 2004/03/06 00:04:38 nickm Exp $
 
 """mixminion.server.Modules
 
@@ -13,6 +13,7 @@ __all__ = [ 'ModuleManager', 'DeliveryModule',
             'DELIVER_OK', 'DELIVER_FAIL_RETRY', 'DELIVER_FAIL_NORETRY'
             ]
 
+import errno
 import os
 import re
 import sys
@@ -64,6 +65,13 @@ class DeliveryModule:
         "Zero-argument constructor, as required by Module protocol."
         pass
 
+    def usesDecodingHandle(self):
+        """Return true iff this module expects to find decoding handles on
+           routinginfo fields.  This should be used by any module that
+           expects to receive replies or forward encrypted messages as
+           described in E2E-spec.txt"""
+        return 1
+
     def getRetrySchedule(self):
         """Return a retry schedule for this module's queue, as specified
            in ServerQueue.DeliveryQueue.setRetrySchedule."""
@@ -438,11 +446,18 @@ class ModuleManager:
 
         mod = self.typeToModule.get(exitType)
         if mod is None:
-            LOG.error("Unable to handle message with unknown type %s",
+            LOG.error("Unable to handle packet with unknown type %s",
                       exitType)
             return "<nil>"
+        try:
+            packet.setTagged(mod.usesDecodingHandle())
+        except ParseError:
+            LOG.error("Packet (type %04x) missing decoding handle; dropped",
+                      exitType)
+            return "<nil>"
+
         queue = self.queues[mod.getName()]
-        LOG.debug("Delivering message %r (type %04x) via module %s",
+        LOG.debug("Delivering packet %r (type %04x) via module %s",
                   packet.getContents()[:8], exitType, mod.getName())
 
         return queue.queueDeliveryMessage(packet)
@@ -499,6 +514,7 @@ class ModuleManager:
 #----------------------------------------------------------------------
 class DropModule(DeliveryModule):
     """Null-object pattern: drops all messages it receives."""
+    def usesDecodingHandle(self): return 0
     def getConfigSyntax(self):
         return { }
     def getRetrySchedule(self):
@@ -542,6 +558,7 @@ class FragmentModule(DeliveryModule):
         self.maxInterval = None
         self.maxFragments = None
         self.lock = threading.RLock()
+    def usesDecodingHandle(self): return 0
     def getConfigSyntax(self):
         return { "Delivery/Fragmented" :
                  { 'Enabled' : ('REQUIRE',  "boolean", "no"),
@@ -560,7 +577,7 @@ class FragmentModule(DeliveryModule):
         deliverySecs = [ 'Delivery/MBOX', 'Delivery/SMTP',
                          'Delivery/SMTP-Via-Mixmaster' ]
         enabled = [ config.get(s,{}).get("Enabled") for s in deliverySecs ]
-        
+
         if not [ e for e in enabled if e ]:
             raise ConfigError("You've specified Fragmented delivery, but no actual delivery method.  This doesn't make much sense.")
 
@@ -624,6 +641,8 @@ class FragmentModule(DeliveryModule):
                 self._queue = None
         finally:
             self.lock.release()
+    def processMessage(self, packet):
+        raise AssertionError
 
 class FragmentDeliveryQueue:
     """Delivery queue for FragmentModule.
@@ -722,6 +741,7 @@ class _FragmentedDeliveryMessage:
         self.tp = None
         self.headers = None
 
+    def setTagged(self,tagged=1): pass
     def isDelivery(self): return 1
     def getExitType(self): return self.exitType
     def getAddress(self): return self.address
@@ -909,7 +929,6 @@ class EmailAddressSet:
         return 0
 
 #----------------------------------------------------------------------
-
 def _cleanMaxSize(sz,modname):
     """Given a 'Maximum-Size' configuration value, ensure that it's at least
        32KB, and round it up to the next highest 1KB increment.  Use 'modname'
@@ -924,6 +943,14 @@ def _cleanMaxSize(sz,modname):
         sz = 1024*kb
     return sz
 
+DEFAULT_SUBJECT_LINE = "Type III Anonymous Message"
+DEFAULT_MBOX_DISCLAIMER = """\
+THIS IS AN ANONYMOUS MESSAGE.  The Mixminion server '%(nickname)s' at
+<%(IP)s has been configured to deliver messages to your address.
+If you do not want to receive messages in the future, contact %(removeaddress)s
+and you will be removed."""
+DEFAULT_SMTP_DISCLAIMER = ""
+
 class MailBase:
     """Implementation class: contains code shared by modules that send email
        messages (such as mbox and smtp)."""
@@ -939,6 +966,19 @@ class MailBase:
     # maxMessageSize: Largest allowable size (after decompression, before
     #   base64) for outgoing messages.
     # allowFromAddr: Boolean: do we support user-supplied from addresses?
+
+    COMMON_OPTIONS = {
+        'MaximumSize' : ('ALLOW', "size", "100K"),
+        'AllowFromAddress' : ('ALLOW', "boolean", "yes"),
+        'SubjectLine' : ('ALLOW', None,
+                         'Type III Anonymous Message'),
+        'X-Abuse' : ('ALLOW', None, None),
+        'Comments' : ('ALLOW', None, None),
+        'Message' : ('ALLOW', None, None),
+        'FromTag' : ('ALLOW', None, "[Anon]"),
+        'ReturnAddress' : ('ALLOW', None, None),
+        }
+
     def _formatEmailMessage(self, address, packet):
         """Given a RFC822 mailbox (delivery address), and an instance of
            DeliveryMessage, return a string containing a message to be sent
@@ -970,11 +1010,37 @@ class MailBase:
 
         # Decode and escape the message, and get ready to send it.
         msg = _escapeMessageForEmail(packet)
-        msg = "To: %s\nFrom: %s\nSubject: %s\n%s%s\n\n%s"%(
+        msg = "To: %s\nFrom: %s\nSubject: %s\n%s%s%s"%(
             address, fromAddr, subject, "".join(morelines), self.header, msg)
 
         return msg
 
+    def initializeHeaders(self, sec):
+        """Sets subject and returns a string that can be added to message
+           headers."""
+        # set subject
+        self.subject = _wrapHeader(sec.get("SubjectLine").strip()).strip()
+
+        header = [ "X-Anonymous: yes\n" ]
+
+        # I'm putting this in a list so adding headers will be simple
+        for h in ['X-Abuse', 'Comments']:
+            val = sec.get(h)
+            if val:
+                header.append(_wrapHeader("%s: %s" % (h, val)))
+
+        # Blank line between headers and body
+        header.append("\n")
+
+        # see if we have a disclaimer
+        disclaimer = sec.get("Message")
+        if disclaimer:
+            header.append("\n".join(textwrap.wrap(disclaimer.strip())))
+            # blank line between disclaimer and body.
+            header.append("\n\n")
+
+        self.header = "".join(header)
+
 #----------------------------------------------------------------------
 class MBoxModule(DeliveryModule, MailBase):
     """Implementation for MBOX delivery: sends messages, via SMTP, to
@@ -1005,18 +1071,15 @@ class MBoxModule(DeliveryModule, MailBase):
     def getConfigSyntax(self):
         # FFFF There should be some way to say that fields are required
         # FFFF if the module is enabled.
-        return { "Delivery/MBOX" :
-                 { 'Enabled' : ('REQUIRE',  "boolean", "no"),
-                   'Retry': ('ALLOW', "intervalList",
-                             "7 hours for 6 days"),
-                   'AddressFile' : ('ALLOW', None, None),
-                   'ReturnAddress' : ('ALLOW', None, None),
-                   'RemoveContact' : ('ALLOW', None, None),
-                   'AllowFromAddress' : ('ALLOW', "boolean", 'yes'),
-                   'SMTPServer' : ('ALLOW', None, 'localhost'),
-                   'MaximumSize' : ('ALLOW', "size", "100K"),
-                   }
-                 }
+        cfg = { 'Enabled' : ('REQUIRE',  "boolean", "no"),
+                'Retry': ('ALLOW', "intervalList",
+                          "7 hours for 6 days"),
+                'AddressFile' : ('ALLOW', "filename", None),
+                'RemoveContact' : ('ALLOW', None, None),
+                'SMTPServer' : ('ALLOW', None, 'localhost'),
+                }
+        cfg.update(MailBase.COMMON_OPTIONS)
+        return { "Delivery/MBOX" : cfg }
 
     def validateConfig(self, config, lines, contents):
         sec = config['Delivery/MBOX']
@@ -1060,15 +1123,8 @@ class MBoxModule(DeliveryModule, MailBase):
                                             "Delivery/MBOX")
 
         # These fields are needed by MailBase
-        self.subject = "Type III Anonymous Message"
+        self.initializeHeaders(sec)
         self.fromTag = "[Anon]"
-        self.header = """\
-X-Anonymous: yes
-
-THIS IS AN ANONYMOUS MESSAGE.  The mixminion server '%s' at
-%s has been configured to deliver messages to your address.
-If you do not want to receive messages in the future, contact %s
-and you will be removed.""" %(self.nickname, self.addr, self.contact)
 
         # Parse the address file.
         self.addresses = {}
@@ -1166,25 +1222,19 @@ class DirectSMTPModule(SMTPModule):
     def __init__(self):
         SMTPModule.__init__(self)
 
+
     def getRetrySchedule(self):
         return self.retrySchedule
 
     def getConfigSyntax(self):
-        return { "Delivery/SMTP" :
-                 { 'Enabled' : ('REQUIRE', "boolean", "no"),
-                   'Retry': ('ALLOW', "intervalList",
-                             "7 hours for 6 days"),
-                   'BlacklistFile' : ('ALLOW', None, None),
-                   'SMTPServer' : ('ALLOW', None, 'localhost'),
-                   'AllowFromAddress': ('ALLOW', "boolean", "yes"),
-                   'Message' : ('ALLOW', None, ""),
-                   'ReturnAddress': ('ALLOW', None, None), #Required on e
-                   'FromTag' : ('ALLOW', None, "[Anon]"),
-                   'SubjectLine' : ('ALLOW', None,
-                                    'Type III Anonymous Message'),
-                   'MaximumSize' : ('ALLOW', "size", "100K"),
-                   }
-                 }
+        cfg = { 'Enabled' : ('REQUIRE', "boolean", "no"),
+                'Retry': ('ALLOW', "intervalList",
+                          "7 hours for 6 days"),
+                'BlacklistFile' : ('ALLOW', "filename", None),
+                'SMTPServer' : ('ALLOW', None, 'localhost'),
+                }
+        cfg.update(MailBase.COMMON_OPTIONS)
+        return { "Delivery/SMTP" : cfg }
 
     def validateConfig(self, config, lines, contents):
         sec = config['Delivery/SMTP']
@@ -1214,15 +1264,11 @@ class DirectSMTPModule(SMTPModule):
             self.blacklist = EmailAddressSet(fname=sec['BlacklistFile'])
         else:
             self.blacklist = None
-        message = "\n".join(textwrap.wrap(sec.get('Message',""))).strip()
-        self.subject = sec['SubjectLine']
         self.returnAddress = sec['ReturnAddress']
         self.fromTag = sec.get('FromTag', "[Anon]")
         self.allowFromAddr = sec['AllowFromAddress']
-        if message:
-            self.header = "X-Anonymous: yes\n\n%s" %(message)
-        else:
-            self.header = "X-Anonymous: yes"
+
+        self.initializeHeaders(sec)
 
         self.maxMessageSize = _cleanMaxSize(sec['MaximumSize'],
                                             "Delivery/SMTP")
@@ -1277,19 +1323,14 @@ class MixmasterSMTPModule(SMTPModule):
         return self.retrySchedule
 
     def getConfigSyntax(self):
-        return { "Delivery/SMTP-Via-Mixmaster" :
-                 { 'Enabled' : ('REQUIRE', "boolean", "no"),
-                   'Retry': ('ALLOW', "intervalList",
-                             "7 hours for 6 days"),
-                   'MixCommand' : ('REQUIRE', "command", None),
-                   'Server' : ('REQUIRE', None, None),
-                   'FromTag' : ('ALLOW', None, "[Anon]"),
-                   'SubjectLine' : ('ALLOW', None,
-                                    'Type III Anonymous Message'),
-                   'MaximumSize' : ('ALLOW', "size", "100K"),
-                   'AllowFromAddress' : ('ALLOW', "boolean", "yes"),
-                   }
-                 }
+        cfg = { 'Enabled' : ('REQUIRE', "boolean", "no"),
+                'Retry': ('ALLOW', "intervalList",
+                          "7 hours for 6 days"),
+                'MixCommand' : ('REQUIRE', "command", None),
+                'Server' : ('REQUIRE', None, None),
+                }
+        cfg.update(MailBase.COMMON_OPTIONS)
+        return { "Delivery/SMTP-Via-Mixmaster" : cfg }
 
     def validateConfig(self, config, lines, contents):
         #FFFF write more
@@ -1305,14 +1346,13 @@ class MixmasterSMTPModule(SMTPModule):
             return
         cmd = sec['MixCommand']
         self.server = sec['Server']
-        self.subject = sec['SubjectLine']
         self.retrySchedule = sec['Retry']
         self.fromTag = sec.get('FromTag', "[Anon]")
         self.allowFromAddr = sec['AllowFromAddress']
         self.command = cmd[0]
         self.options = tuple(cmd[1]) + ("-l", self.server)
         self.returnAddress = "nobody"
-        self.header = "X-Anonymous: yes"
+        self.initializeHeaders(sec)
         self.maxMessageSize = _cleanMaxSize(sec['MaximumSize'],
                                             "Delivery/SMTP-Via-Mixmaster")
         manager.enableModule(self)
@@ -1346,7 +1386,13 @@ class MixmasterSMTPModule(SMTPModule):
 
         cmd = self.command
         opts = self.options + (self.tmpQueue.getMessagePath(handle),)
-        code = os.spawnl(os.P_WAIT, cmd, cmd, *opts)
+        try:
+            code = os.spawnl(os.P_WAIT, cmd, cmd, *opts)
+        except OSError,e:
+            if e.errno not in (errno.EAGAIN, errno.ENOMEM): raise
+            LOG.warn("Transient error while running Mixmaster: %s",e)
+            return DELIVER_FAIL_RETRY
+
         LOG.debug("Queued Mixmaster message: exit code %s", code)
         self.tmpQueue.removeMessage(handle)
         return DELIVER_OK
@@ -1356,7 +1402,12 @@ class MixmasterSMTPModule(SMTPModule):
            should be called after invocations of processMessage."""
         cmd = self.command
         LOG.debug("Flushing Mixmaster pool")
-        os.spawnl(os.P_WAIT, cmd, cmd, "-S")
+        try:
+            os.spawnl(os.P_WAIT, cmd, cmd, "-S")
+        except OSError,e:
+            if e.errno not in (errno.EAGAIN, errno.ENOMEM): raise
+            LOG.warn("Transient error while running Mixmaster: %s",e)
+            return DELIVER_FAIL_RETRY
 
 class _MixmasterSMTPModuleDeliveryQueue(SimpleModuleDeliveryQueue):
     """Delivery queue for _MixmasterSMTPModule.  Same as
@@ -1374,7 +1425,6 @@ def checkMailHeaders(headers):
        for an outgoing email message.  Raise ParseError if they are not."""
     for k in headers.keys():
         if k not in MAIL_HEADERS:
-            #XXXX this should raise parse error instead.
             LOG.warn("Skipping unrecognized mail header %s"%k)
 
     fromAddr = headers['FROM']
@@ -1409,6 +1459,13 @@ def sendSMTPMessage(server, toList, fromAddr, message):
 
 #----------------------------------------------------------------------
 
+def _wrapHeader(text):
+    """Wraps a header. If it's more than 70 characters long, adds
+       spaces to the next lines so it's properly indented."""
+    lines = textwrap.wrap(text.strip(), width=70, subsequent_indent="    ")
+    lines.append("") # to get final \n
+    return "\n".join(lines)
+
 def _escapeMessageForEmail(packet):
     """Helper function: Given a DeliveryPacket, escape the message if
        it is not plaintext ascii, and wrap it in some standard
