@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: PacketHandler.py,v 1.34 2003/12/08 02:22:56 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: PacketHandler.py,v 1.38 2004/03/07 06:31:46 nickm Exp $
 
 """mixminion.server.PacketHandler: Code to process mixminion packets"""
 
@@ -10,15 +10,14 @@ import types
 from mixminion.Common import encodeBase64, formatBase64, LOG
 import mixminion.Crypto as Crypto
 import mixminion.Packet as Packet
-import mixminion.Common as Common
 import mixminion.BuildMessage
 
 from mixminion.ServerInfo import PACKET_KEY_BYTES
-from mixminion.Common import MixError, isPrintingAscii
+from mixminion.Common import MixError, MixFatalError, isPrintingAscii
 
 __all__ = [ 'PacketHandler', 'ContentError', 'DeliveryPacket', 'RelayedPacket']
 
-class ContentError(Common.MixError):
+class ContentError(MixError):
     """Exception raised when a packed is malformatted or unacceptable."""
     pass
 
@@ -61,7 +60,7 @@ class PacketHandler:
             for k in keys:
                 newKeys[k.encode_key(1)] = 1
                 if k.get_modulus_bytes() != PACKET_KEY_BYTES:
-                    raise Common.MixFatalError("Incorrect packet key length")
+                    raise MixFatalError("Incorrect packet key length")
             # For all old public keys, if they aren't in the new set, close
             # their hashlogs.
             for k, h in self.privatekeys:
@@ -206,9 +205,9 @@ class PacketHandler:
         # If we're an exit node, there's no need to process the headers
         # further.
         if rt >= Packet.MIN_EXIT_TYPE:
-            return DeliveryPacket(rt, subh.getExitAddress(),
+            return DeliveryPacket(rt, subh.getExitAddress(0),
                                   keys.get(Crypto.APPLICATION_KEY_MODE),
-                                  subh.getTag(), payload)
+                                  payload)
 
         # If we're not an exit node, make sure that what we recognize our
         # routing type.
@@ -289,18 +288,15 @@ class DeliveryPacket:
     # dPayload -- An instance of mixminion.Packet.Payload for this object.
     # error -- None, or a string containing an error encountered while trying
     #     to decode the payload.
-    def __init__(self, routingType, routingInfo, applicationKey,
-                 tag, payload):
+    def __init__(self, routingType, routingInfo, applicationKey, payload):
         """Construct a new DeliveryPacket."""
         assert 0 <= routingType <= 0xFFFF
         assert len(applicationKey) == 16
-        #assert len(tag) == 20 #XXXX007 make tag system sane.
-        assert len(tag) == 20 or routingType == Packet.FRAGMENT_TYPE
         assert len(payload) == 28*1024
         self.exitType = routingType
         self.address = routingInfo
         self.key = applicationKey
-        self.tag = tag
+        self.tag = ""
         self.payload = payload
         self.contents = None
         self.type = None
@@ -309,26 +305,34 @@ class DeliveryPacket:
         self.dPayload = None
         self.error = None
 
+    def setTagged(self,tagged=1):
+        """Re-frame the routingInfo in this packet. If 'tagged' is true,
+           then the routingInfo starts with TAG_LEN bytes of decoding
+           handle, and the rest is address.  If 'tagged' is false, then
+           it's all address.
+        """
+        x = self.tag+self.address
+        if tagged:
+            if len(x)<Packet.TAG_LEN:
+                raise Packet.ParseError("Missing decoding handle for exit type")
+            self.tag = x[:Packet.TAG_LEN]
+            self.address = x[Packet.TAG_LEN:]
+        else:
+            self.tag = ""
+            self.address = x
+
     def __getstate__(self):
         return "V0", self.__dict__
 
     def __setstate__(self, state):
-        if type(state) == types.DictType:
-            #XXXX007 remove this case.  (Not used since 0.0.5alpha)
-            LOG.warn("Found ancient packet format.")
-            self.__dict__.update(state)
-            if not hasattr(self, 'isfrag'):
-                self.isfrag = 0
-            if not hasattr(self, 'dPayload'):
-                self.dPayload = None
-            if not hasattr(self, 'error'):
-                self.error = None
-            if not hasattr(self, 'headers'):
-                self.headers = {}
-        elif state[0] == 'V0':
-            self.__dict__.update(state[1])
+        if type(state) == types.TupleType:
+            if state[0] == 'V0':
+                self.__dict__.update(state[1])
+            else:
+                raise MixError("Unrecognized state version %s" % state[0])
         else:
-            raise MixError("Unrecognized state version %s" % state[0])
+            raise MixError("Unrecognized state type %s"% type(state))
+
 
     def isDelivery(self):
         """Return true iff this packet is a delivery (non-relay) packet."""
@@ -390,8 +394,7 @@ class DeliveryPacket:
         message = self.payload
         self.contents = None
         try:
-            self.dPayload = mixminion.BuildMessage.decodePayload(message,
-                                                                 self.tag)
+            self.dPayload = mixminion.BuildMessage.decodePayload(message, "")
             if self.dPayload is None:
                 # encrypted message
                 self.type = 'enc'
