@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: ServerKeys.py,v 1.61 2003/12/03 23:18:53 nickm Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: ServerKeys.py,v 1.66 2004/03/06 00:04:38 nickm Exp $
 
 """mixminion.ServerKeys
 
@@ -422,7 +422,7 @@ class ServerKeyring:
         #     * it did not become invalid until keyOverlap seconds ago
 
         return [ (va,vu,k) for (va,vu,k) in self.keySets
-                 if va < now and vu > cutoff ]
+                 if va <= now and vu >= cutoff ]
 
     def getServerKeysets(self, now=None):
         """Return list of ServerKeyset objects for the currently live keys.
@@ -433,9 +433,6 @@ class ServerKeyring:
             ks.load()
             keysets.append(ks)
 
-        if len(keysets) > 2:
-            LOG.error("Got >2 active keys! That's not supposed to happen.")
-
         return keysets
 
     def _getDHFile(self):
@@ -471,7 +468,7 @@ class ServerKeyring:
            This function is idempotent.
         """
         self.checkKeys()
-        deadKeys = self.getDeadKeys()
+        deadKeys = self.getDeadKeys(when)
         self.currentKeys = keys = self.getServerKeysets(when)
         keyNames = [k.keyname for k in keys]
         deadKeyNames = [k.keyname for msg, k in deadKeys]
@@ -479,6 +476,9 @@ class ServerKeyring:
                  len(keys), " ".join(keyNames),
                  len(deadKeys), " ".join(deadKeyNames))
         if mmtpServer is not None:
+            LOG.trace("Using TLS cert from %s: good from %s to %s",
+                      keyNames[-1], formatDate(keys[-1].validAfter),
+                      formatDate(keys[-1].validUntil))
             context = self._getTLSContext(keys[-1])
             mmtpServer.setServerContext(context)
         if packetHandler is not None:
@@ -874,9 +874,10 @@ def checkDescriptorConsistency(info, config, log=1, isPublished=1):
 
     info_testing = info.get("Testing",{})
     if info_testing.get("Platform", "") != getPlatformSummary():
-        warn("Mismatched platform summary")
+        warn("Mismatched platform: running %r, but %r published",
+             getPlatformSummary(), info_testing.get("Platform",""))
     if not warn.errors and info_testing.get("Configuration", "") != config.getConfigurationSummary():
-        warn("Mismatched configuration summary")
+        warn("Configuration has changed since last publication")
 
     if warn.errors:
         return "bad"
@@ -968,7 +969,7 @@ def generateServerDescriptorAndKeys(config, identityKey, keydir, keyname,
     mmtpProtocolsIn = ",".join(mmtpProtocolsIn)
     mmtpProtocolsOut = ",".join(mmtpProtocolsOut)
 
-    #XXXX007 remove
+    #XXXX008 remove: hasn't been needed since 0.0.5.
     identityKeyID = formatBase64(
                       mixminion.Crypto.sha1(
                           mixminion.Crypto.pk_encode_public_key(identityKey)))
@@ -988,14 +989,14 @@ def generateServerDescriptorAndKeys(config, identityKey, keydir, keyname,
         "KeyID": identityKeyID,
         "MMTPProtocolsIn" : mmtpProtocolsIn,
         "MMTPProtocolsOut" : mmtpProtocolsOut,
-        "PacketFormat" : "%s.%s"%(mixminion.Packet.MAJOR_NO,
-                                  mixminion.Packet.MINOR_NO),
+        "PacketVersion" : mixminion.Packet.PACKET_VERSION,
         "mm_version" : mixminion.__version__,
-        "Secure" : secure
+        "Secure" : secure,
+        "Contact" : contact,
         }
 
     # If we don't know our IP address, try to guess
-    if fields['IP'] == '0.0.0.0': #XXXX007 remove
+    if fields['IP'] == '0.0.0.0': #XXXX008 remove; not needed since 005.
         try:
             fields['IP'] = _guessLocalIP()
             LOG.warn("No IP configured; guessing %s",fields['IP'])
@@ -1031,14 +1032,13 @@ def generateServerDescriptorAndKeys(config, identityKey, keydir, keyname,
         Valid-After: %(ValidAfter)s
         Valid-Until: %(ValidUntil)s
         Packet-Key: %(PacketKey)s
-        Packet-Versions: %(PacketFormat)s
+        Packet-Versions: %(PacketVersion)s
         Software: Mixminion %(mm_version)s
         Secure-Configuration: %(Secure)s
+        Contact: %(Contact)s
         """ % fields
     if insecurities:
         info += "Why-Insecure: %s\n"%(", ".join(insecurities))
-    if contact:
-        info += "Contact: %s\n"%contact
     if comments:
         info += "Comments: %s\n"%comments
 
