@@ -1,5 +1,5 @@
-# Copyright 2002-2003 Nick Mathewson.  See LICENSE for licensing information.
-# $Id: ServerMain.py,v 1.111 2003/12/14 02:15:20 weasel Exp $
+# Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information.
+# $Id: ServerMain.py,v 1.124.2.1 2004/04/26 16:57:18 nickm Exp $
 
 """mixminion.ServerMain
 
@@ -20,6 +20,7 @@
 #                        hash_2*     corresponding to key sets]
 #                          ...
 #               stats.tmp         [Cache of stats from latest period]
+#               dir/...           [Directory dowloaded from directory server.]
 #
 #    QUEUEDIR defaults to ${WORKDIR}/queues
 #    ${QUEUEDIR}/incoming/        [Queue of received,unprocessed pkts]
@@ -50,11 +51,12 @@ import signal
 import time
 import threading
 from types import *
-# We pull this from mixminion.Common, just in case somebody still has
+# We pull this from mixminion.ThreadUtils just in case somebody still has
 # a copy of the old "mixminion/server/Queue.py" (since renamed to
 # ServerQueue.py)
-from mixminion.Common import MessageQueue, ClearableQueue, QueueEmpty
+from mixminion.ThreadUtils import MessageQueue, ClearableQueue, QueueEmpty
 
+import mixminion.ClientDirectory
 import mixminion.Config
 import mixminion.Crypto
 import mixminion.Filestore
@@ -71,7 +73,7 @@ from bisect import insort
 from mixminion.Common import LOG, LogStream, MixError, MixFatalError,\
      UIError, ceilDiv, createPrivateDir, disp64, formatTime, \
      installSIGCHLDHandler, Lockfile, LockfileLocked, readFile, secureDelete, \
-     tryUnlink, waitForChildren, writeFile
+     succeedingMidnight, tryUnlink, waitForChildren, writeFile
 
 # Version number for server home-directory.
 #
@@ -129,7 +131,7 @@ def checkHomedirVersion(config):
             print >>sys.stderr, """\
 This server's files are stored in an older format, and are not compatible
 with this version of the mixminion server.  To upgrade, run:
-     'mixminion server-upgrade'."""
+     'mixminiond upgrade'."""
             raise UIError
         else:
             print >>sys.stderr, """\
@@ -364,7 +366,7 @@ class OutgoingQueue(mixminion.server.ServerQueue.DeliveryQueue):
                 for pending in packets ]
             LOG.trace("Delivering packets OUT:[%s] to %s",
                       " ".join([p.getHandle() for p in packets]),
-                      routing)
+                      mixminion.ServerInfo.displayServerByRouting(routing))
             self.server.sendPacketsByRouting(routing, deliverable)
 
 class _MMTPServer(mixminion.server.MMTPServer.MMTPAsyncServer):
@@ -396,8 +398,8 @@ class CleaningThread(threading.Thread):
        main thread.
     """
     # Fields:
-    #   mqueue: A ClearableQueue holding filenames to delete, or None to indicate
-    #     a shutdown.
+    #   mqueue: A ClearableQueue holding lists of filenames to delete,
+    #     or None to indicate a shutdown.
     def __init__(self):
         threading.Thread.__init__(self)
         self.mqueue = ClearableQueue()
@@ -406,12 +408,11 @@ class CleaningThread(threading.Thread):
         """Schedule the file named 'fname' for deletion"""
         #LOG.trace("Scheduling %s for deletion", fname)
         assert fname is not None
-        self.mqueue.put(fname)
+        self.mqueue.put([fname])
 
     def deleteFiles(self, fnames):
         """Schedule all the files in the list 'fnames' for deletion"""
-        for f in fnames:
-            self.deleteFile(f)
+        self.mqueue.put(fnames)
 
     def shutdown(self):
         """Tell this thread to shut down once it has deleted all pending
@@ -425,16 +426,32 @@ class CleaningThread(threading.Thread):
            a filename to delete or an indication to shutdown, then
            acts accordingly."""
         try:
-            while 1:
-                fn = self.mqueue.get()
-                if fn is None:
-                    LOG.info("Cleanup thread shutting down.")
-                    return
-                if os.path.exists(fn):
-                    #LOG.trace("Deleting %s", fn)
-                    secureDelete(fn, blocking=1)
-                else:
-                    LOG.warn("Delete thread didn't find file %s",fn)
+            running = 1
+            while running:
+                fnames = self.mqueue.get()
+                if fnames is None:
+                    fnames = []
+                    running = 0
+
+                try:
+                    while 1:
+                        more = self.mqueue.get(0)
+                        if more is None:
+                            running=0
+                        fnames.extend(more)
+                except QueueEmpty:
+                    pass
+
+                delNames = []
+                for fn in fnames:
+                    if os.path.exists(fn):
+                        delNames.append(fn)
+                    else:
+                        LOG.warn("Delete thread didn't find file %s",fn)
+
+                secureDelete(delNames, blocking=1)
+
+            LOG.info("Cleanup thread shutting down.")
         except:
             LOG.error_exc(sys.exc_info(),
                           "Exception while cleaning; shutting down thread.")
@@ -653,17 +670,6 @@ class MixminionServer(_Scheduler):
             writeFile(os.path.join(homeDir, "version"),
                       SERVER_HOMEDIR_VERSION, 0644)
 
-        # Obsolete lock file.
-        #XXXX007: remove this check.
-        lockFname = os.path.join(homeDir, "lock")
-        if os.path.exists(lockFname):
-            lf = Lockfile(lockFname)
-            try:
-                lf.acquire()
-                lf.release()
-            except LockfileLocked:
-                raise UIError("Another (older) server seems to be running")
-
         # The pid/lock file.
         self.pidFile = config.getPidFile()
         if not os.path.exists(os.path.split(self.pidFile)[0]):
@@ -671,7 +677,7 @@ class MixminionServer(_Scheduler):
             os.makedirs(os.path.split(self.pidFile)[0], 0700)
         self.lockFile = Lockfile(self.pidFile)
         try:
-            self.lockFile.acquire(mode=0644)
+            self.lockFile.acquire(mode=0644,blocking=0)
         except LockfileLocked:
             raise UIError("Another server seems to be running.")
 
@@ -682,7 +688,7 @@ class MixminionServer(_Scheduler):
 
         # Try to read the keyring.  If we have a pre-0.0.4 version of
         # mixminion, we might have some bad server descriptors lying
-        # around.  If so, tell the user to run server-upgrade.
+        # around.  If so, tell the user to run 'mixminiond upgrade.'
         try:
             self.keyring = mixminion.server.ServerKeys.ServerKeyring(config)
         except mixminion.Config.ConfigError, e:
@@ -697,8 +703,8 @@ class MixminionServer(_Scheduler):
             else:
                 raise UIError((
 "For some reason, your generated server descriptors cannot be parsed.  You\n"
-"may want to delete all your keysets with server-DELKEYS and have the server\n"
-"generate new ones.  [Messages sent to the old keys will be lost].\n"
+"may want to delete all your keysets with mixminiond DELKEYS and have the\n"
+"server generate new ones.  [Messages sent to the old keys will be lost].\n"
 "The original error message was '%s'.")%e)
 
         self.keyring.removeDeadKeys()
@@ -716,6 +722,19 @@ class MixminionServer(_Scheduler):
         self.descriptorFile = os.path.join(homeDir, "current-desc")
         self.keyring.updateKeys(self.packetHandler, self.mmtpServer,
                                 self.descriptorFile)
+        LOG.debug("Initializing directory client")
+        self.dirClient = mixminion.ClientDirectory.ClientDirectory(
+            os.path.join(config.getWorkDir(),"dir"))
+        try:
+            self.dirClient.updateDirectory()
+        except mixminion.ClientDirectory.GotInvalidDirectoryError, e:
+            LOG.warn(str(e))
+            LOG.warn("   (I'll use the old one until I get one that's good.)")
+        except UIError, e:#XXXX008 This should really be a new exception
+            LOG.warn(str(e))
+            LOG.warn("   (I'll use the old one until a download succeeds.)")
+
+        self.dirClient._installAsKeyIDResolver()
 
         publishedIP, publishedPort, publishedKeyID = self.keyring.getAddress()
 
@@ -813,6 +832,29 @@ class MixminionServer(_Scheduler):
 
         self.processingThread.addJob(c)
 
+    def updateDirectoryClient(self):
+        def c(self=self):
+            try:
+                self.dirClient.updateDirectory()
+                nextUpdate = succeedingMidnight(time.time()+30)
+                prng = mixminion.Crypto.getCommonPRNG()
+                # Randomly retrieve the directory within an hour after
+                # midnight, to avoid hosing the server.
+                nextUpdate += prng.getInt(60)*60
+            except mixminion.ClientDirectory.GotInvalidDirectoryError, e:
+                LOG.warn(str(e))
+                LOG.warn("    I'll try again in an hour.")
+                nextUpdate = min(succeedingMidnight(time.time()+30),
+                                 time.time()+3600)
+            except UIError, e:#XXXX008 This should really be a new exception
+                LOG.warn(str(e))
+                LOG.warn("    I'll try again in an hour.")
+                nextUpdate = min(succeedingMidnight(time.time()+30),
+                                 time.time()+3600)
+            self.scheduleOnce(nextUpdate, "UPDATE_DIR_CLIENT",
+                              self.updateDirectoryClient)
+        self.processingThread.addJob(c)
+
     def run(self):
         """Run the server; don't return unless we hit an exception."""
         global GOT_HUP
@@ -853,6 +895,9 @@ class MixminionServer(_Scheduler):
                           "KEY_GEN",
                           self.generateKeys)
 
+        # Makes next update get scheduled.
+        self.updateDirectoryClient()
+
         nextMix = self.mixPool.getNextMixTime(now)
         LOG.debug("First mix at %s", formatTime(nextMix,1))
         self.scheduleRecurringComplex(self.mixPool.getNextMixTime(now),
@@ -870,9 +915,11 @@ class MixminionServer(_Scheduler):
             nextEventTime = self.firstEventTime()
             now = time.time()
             timeLeft = nextEventTime - now
+            tickInterval = self.mmtpServer.TICK_INTERVAL
+            nextTick = now+tickInterval
             while timeLeft > 0:
                 # Handle pending network events
-                self.mmtpServer.process(2)
+                self.mmtpServer.process(tickInterval)
                 # Check for signals
                 if STOPPING:
                     LOG.info("Caught SIGTERM; shutting down.")
@@ -890,6 +937,9 @@ class MixminionServer(_Scheduler):
 
                 # Calculate remaining time until the next event.
                 now = time.time()
+                if now > nextTick:
+                    self.mmtpServer.tick()
+                    nextTick = now+tickInterval
                 timeLeft = nextEventTime - now
 
             # An event has fired.
@@ -1001,10 +1051,13 @@ def closeUnusedFDs():
     """Close stdin, stdout, and stderr."""
     # (We could try to do this via sys.stdin.close() etc., but then we
     #  would miss the magic copies in sys.__stdin__, sys.__stdout__, etc.
-    #  Using os.close instead just nukes the FD for us.)
-    os.close(sys.stdin.fileno())
-    os.close(sys.stdout.fileno())
-    os.close(sys.stderr.fileno())
+    #  Using os.dup2 instead just nukes the old file for us, and keeps the
+    #  fd from getting reused.)
+    nullfd = os.open("/dev/null", os.O_RDWR|os.O_APPEND)
+    os.dup2(nullfd, sys.stdin.fileno())
+    os.dup2(nullfd, sys.stdout.fileno())
+    os.dup2(nullfd, sys.stderr.fileno())
+    os.close(nullfd)
     # Override stdout and stderr in case some code tries to use them
     sys.stdout = sys.__stdout__ = LogStream("STDOUT", "WARN")
     sys.stderr = sys.__stderr__ = LogStream("STDERR", "WARN")
@@ -1027,11 +1080,12 @@ def configFromServerArgs(cmd, args, usage):
                                    "daemon", "nodaemon", "echo", "severity="])
     if args:
         print >>sys.stderr, "No arguments expected."
+        if len(args) == 1:
+            print >>sys.stderr, "Did you mean to use the '-f' flag?"
         print usage
-        sys.exit(0)
+        sys.exit(1)
     configFile = None
     forceDaemon = None
-    echo = 0
     severity = None
     for o,v in options:
         if o in ('-h', '--help'):
@@ -1060,7 +1114,7 @@ def configFromServerArgs(cmd, args, usage):
         _ECHO_OPT = 1
     elif _QUIET_OPT:
         # Don't even say we're silencing the log.
-        mixminion.Common.LOG.silenceNoted = 1 
+        mixminion.Common.LOG.silenceNoted = 1
         config['Server']['EchoMessages'] = 0
     if forceDaemon is not None:
         config['Server']['Daemon'] = forceDaemon
@@ -1082,6 +1136,7 @@ def readConfigFile(configFile):
             p = os.path.expanduser(p)
             if os.path.exists(p):
                 configFile = p
+                break
         if configFile is None:
             print >>sys.stderr, "No config file found or specified."
             sys.exit(1)
@@ -1102,7 +1157,7 @@ def readConfigFile(configFile):
 
 #----------------------------------------------------------------------
 _SERVER_START_USAGE = """\
-Usage: mixminion server-start [options]
+Usage: mixminiond start [options]
 Start a Mixminion server.
 Options:
   -h, --help:                Print this usage message and exit.
@@ -1117,7 +1172,7 @@ Options:
 def runServer(cmd, args):
     """[Entry point]  Start a Mixminion server."""
     if cmd.endswith(" server"):
-        print "Obsolete command. Use 'mixminion server-start' instead."
+        print "Obsolete command. Use 'mixminiond start' instead."
 
     config = configFromServerArgs(cmd, args, _SERVER_START_USAGE)
     checkHomedirVersion(config)
@@ -1195,7 +1250,7 @@ def runServer(cmd, args):
 
 #----------------------------------------------------------------------
 _UPGRADE_USAGE = """\
-Usage: mixminion server-upgrade [options]
+Usage: mixminiond upgrade [options]
 Upgrade the server's home directory from an earlier version.
 Options:
   -h, --help:                Print this usage message and exit.
@@ -1283,7 +1338,7 @@ def runUpgrade(cmd, args):
 
 #----------------------------------------------------------------------
 _DELKEYS_USAGE = """\
-Usage: mixminion server-DELKEYS [options]
+Usage: mixminiond DELKEYS [options]
 Delete all keys for this server (except the identity key).
 Options:
   -h, --help:                Print this usage message and exit.
@@ -1303,7 +1358,6 @@ def runDELKEYS(cmd, args):
 
     checkHomedirVersion(config)
 
-    homeDir = config.getBaseDir()
     keyDir = config.getKeyDir()
     hashDir = os.path.join(config.getWorkDir(), 'hashlogs')
     if not os.path.exists(keyDir):
@@ -1321,7 +1375,7 @@ def runDELKEYS(cmd, args):
 
 #----------------------------------------------------------------------
 _PRINT_STATS_USAGE = """\
-Usage: mixminion server-stats [options]
+Usage: mixminiond stats [options]
 Print server statistics for the current statistics interval.
 Options:
   -h, --help:                Print this usage message and exit.
@@ -1339,7 +1393,7 @@ def printServerStats(cmd, args):
 
 #----------------------------------------------------------------------
 _SIGNAL_SERVER_USAGE = """\
-Usage: mixminion %s [options]
+Usage: mixminiond %s [options]
 Tell a mixminion server to %s.
 Options:
   -h, --help:                Print this usage message and exit.
@@ -1349,13 +1403,13 @@ Options:
 def signalServer(cmd, args):
     """[Entry point] Send a SIGHUP or a SIGTERM to a running mixminion
        server."""
-    if cmd.endswith("server-stop"):
-        reload = 0
-        usage = _SIGNAL_SERVER_USAGE % ("server-stop", "shut down")
+    if cmd.endswith("stop"):
+        sig_reload = 0
+        usage = _SIGNAL_SERVER_USAGE % ("stop", "shut down")
     else:
-        assert cmd.endswith("server-reload")
-        reload = 1
-        usage = _SIGNAL_SERVER_USAGE % ("server-reload",
+        assert cmd.endswith("reload")
+        sig_reload = 1
+        usage = _SIGNAL_SERVER_USAGE % ("reload",
                                         "rescan its configuration")
 
     config = configFromServerArgs(cmd, args, usage=usage)
@@ -1363,14 +1417,13 @@ def signalServer(cmd, args):
 
     checkHomedirVersion(config)
 
-    _signalServer(config, reload)
+    _signalServer(config, sig_reload)
 
 def _signalServer(config, reload):
     """Given a configuration file, sends a signal to the corresponding
        server if it's running.  If 'reload', the signal is HUP.  Else,
        the signal is TERM.
     """
-    homeDir = config.getBaseDir()
     pidFile = config.getPidFile()
     if not os.path.exists(pidFile):
         raise UIError("No server seems to be running.")
@@ -1396,7 +1449,7 @@ def _signalServer(config, reload):
 
 #----------------------------------------------------------------------
 _REPUBLISH_USAGE = """\
-Usage: mixminion server-republish [options]
+Usage: mixminiond republish [options]
 Force a mixminion server to republish its keys to the directory.
 Options:
   -h, --help:                Print this usage message and exit.
