@@ -1,5 +1,5 @@
-/* Copyright (c) 2002-2003 Nick Mathewson.  See LICENSE for licensing information */
-/* $Id: tls.c,v 1.31 2003/11/28 04:14:05 nickm Exp $ */
+/* Copyright 2002-2004 Nick Mathewson.  See LICENSE for licensing information*/
+/* $Id: tls.c,v 1.35 2004/03/06 00:04:38 nickm Exp $ */
 #include "_minionlib.h"
 
 #include <time.h>
@@ -198,6 +198,9 @@ mm_TLSContext_new(PyObject *self, PyObject *args, PyObject *kwargs)
         }
         if (!err)
                 SSL_CTX_set_verify(ctx, SSL_VERIFY_NONE, NULL);
+        if (!err)
+                SSL_CTX_set_mode(ctx, SSL_MODE_ENABLE_PARTIAL_WRITE|
+                                      SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
         Py_END_ALLOW_THREADS
 
         if (!err) {
@@ -612,7 +615,6 @@ mm_TLSSock_check_cert_alive(PyObject *self, PyObject *args, PyObject *kwargs)
         return NULL;
 }
 
-
 static char mm_TLSSock_verify_cert_and_get_identity_pk__doc__[] =
   "verify_cert_and_get_identity_pk()\n\n"
   "Check whether all of the following conditions hold:\n"
@@ -740,23 +742,23 @@ mm_TLSSock_do_handshake(PyObject *self, PyObject *args, PyObject *kwargs)
         return Py_None;
 }
 
-#if 0
-static char mm_TLSSock_get_num_renegotiations__doc__[] =
-"tlssock.get_num_renegotiations()\n\n";
+static char mm_TLSSock_get_num_bytes_raw__doc__[] =
+"tlssock.get_num_bytes_raw()\n\n"
+"Return a total number of bytes read and written for this TLS connection\n";
+
 
 static PyObject*
-mm_TLSSock_get_num_renegotiations(PyObject *self, PyObject *args,
-                                  PyObject *kwargs)
+mm_TLSSock_get_num_bytes_raw(PyObject *self, PyObject* args, PyObject *kwargs)
 {
         SSL *ssl;
-
+        unsigned long r, w;
         assert(mm_TLSSock_Check(self));
         FAIL_IF_ARGS();
         ssl = ((mm_TLSSock*)self)->ssl;
-
-        return PyInt_FromLong(SSL_num_renegotiations(ssl));
+        r = BIO_number_read(SSL_get_rbio(ssl));
+        w = BIO_number_written(SSL_get_wbio(ssl));
+        return PyInt_FromLong((long)(r+w));
 }
-#endif
 
 static PyMethodDef mm_TLSSock_methods[] = {
         METHOD(mm_TLSSock, accept),
@@ -771,9 +773,7 @@ static PyMethodDef mm_TLSSock_methods[] = {
         METHOD(mm_TLSSock, fileno),
         METHOD(mm_TLSSock, do_handshake),
         METHOD(mm_TLSSock, renegotiate),
-#if 0
-        METHOD(mm_TLSSock, get_num_renegotiations),
-#endif
+        METHOD(mm_TLSSock, get_num_bytes_raw),
         { NULL, NULL }
 };
 
